=== modified file 'Makefile'
--- Makefile	2006-12-24 06:43:14 +0000
+++ Makefile	2007-10-29 16:19:30 +0000
@@ -7,3 +7,6 @@
 clean:
 	rm -rf dist/
 
+check:
+	# XXX MichaelHudson 2007-10-29 bug=158361: We can't run the loggerhead
+	# tests in PQM yet :(

=== modified file 'dev.cfg'
--- dev.cfg	2007-01-14 05:40:40 +0000
+++ dev.cfg	2007-12-12 08:36:51 +0000
@@ -36,6 +36,10 @@
 server.environment="production"
 #autoreload.package="loggerhead"
 
+# We do *not* want our query arguments turned into unicode strings (as
+# we use them for revision/file ids, and these must not be unicode).
+decoding_filter.on = False
+
 # session_filter.on = True
 
 # Set to True if you'd like to abort execution if a controller gets an

=== modified file 'loggerhead/__init__.py'
--- loggerhead/__init__.py	2007-07-04 14:51:40 +0000
+++ loggerhead/__init__.py	2006-12-11 06:44:19 +0000
@@ -1,1 +0,0 @@
-import turbozpt

=== modified file 'loggerhead/branchview.py'
--- loggerhead/branchview.py	2007-03-06 04:27:05 +0000
+++ loggerhead/branchview.py	2008-02-27 03:36:09 +0000
@@ -29,7 +29,7 @@
 from cherrypy import HTTPRedirect
 
 from loggerhead import util
-from loggerhead.changecache import ChangeCache
+from loggerhead.changecache import ChangeCache, FileChangeCache
 from loggerhead.history import History
 from loggerhead.textindex import TextIndex
 from loggerhead.controllers.changelog_ui import ChangeLogUI
@@ -53,12 +53,12 @@
         self._config = config
         self._project_config = project_config
         self.log = logging.getLogger('loggerhead.%s' % (name,))
-        
+
         # branch history
         self._history_lock = threading.RLock()
         self._history = None
         self._closed = False
-        
+
         self.changes = ChangeLogUI(self)
         self.revision = RevisionUI(self)
         self.files = InventoryUI(self)
@@ -66,12 +66,12 @@
         self.download = DownloadUI(self)
         self.atom = AtomUI(self)
         self.bundle = BundleUI(self)
-        
+
         # force history object to be loaded:
         self.get_history()
-        
+
         turbogears.startup.call_on_shutdown.append(self.close)
-    
+
     @with_history_lock
     def close(self):
         # it's important that we cleanly detach the history, so the cache
@@ -80,13 +80,13 @@
         self._history.detach()
         self._history = None
         self._closed = True
-            
+
     config = property(lambda self: self._config)
-    
+
     name = property(lambda self: self._name)
 
     group_name = property(lambda self: self._group_name)
-    
+
     def _get_friendly_name(self):
         name = self._config.get('branch_name', None)
         if name is not None:
@@ -106,9 +106,9 @@
         # try branch-specific config?
         description = self.get_history().get_config().get_user_option('description')
         return description
-        
+
     description = property(_get_description)
-    
+
     def _get_branch_url(self):
         url = self._config.get('url', None)
         if url is not None:
@@ -120,9 +120,9 @@
         # try branch-specific config?
         url = self.get_history().get_config().get_user_option('public_branch')
         return url
-        
+
     branch_url = property(_get_branch_url)
-    
+
     @turbogears.expose()
     def index(self):
         raise HTTPRedirect(self.url('/changes'))
@@ -141,22 +141,25 @@
             self.log.debug('Reload branch history...')
             if self._history is not None:
                 self._history.detach()
-            self._history = History.from_folder(self._absfolder, self._name)
+            _history = self._history = History.from_folder(
+                self._absfolder, self._name)
             cache_path = self._config.get('cachepath', None)
             if cache_path is None:
                 # try the project config
                 cache_path = self._project_config.get('cachepath', None)
             if cache_path is not None:
-                self._history.use_cache(ChangeCache(self._history, cache_path))
-                self._history.use_search_index(TextIndex(self._history, cache_path))
+                _history.use_cache(ChangeCache(_history, cache_path))
+                _history.use_file_cache(FileChangeCache(_history, cache_path))
+                _history.use_search_index(TextIndex(_history, cache_path))
         return self._history
-    
+
     def check_rebuild(self):
         h = self.get_history()
         if h is not None:
             h.check_rebuild()
-    
+
     def url(self, elements, **kw):
+        "build an url relative to this branch"
         if not isinstance(elements, list):
             elements = [elements]
         if elements[0].startswith('/'):
@@ -164,6 +167,10 @@
         elements = [urllib.quote(x) for x in elements]
         return turbogears.url([ '/' + self.group_name, self.name ] + elements, **kw)
 
+    def context_url(self, elements, **kw):
+        "build an url relative to this branch, bringing along browsing context"
+        return self.url(elements, **util.get_context(**kw))
+
     def last_updated(self):
         h = self.get_history()
         change = h.get_changes([ h.last_revid ])[0]

=== modified file 'loggerhead/changecache.py'
--- loggerhead/changecache.py	2007-06-25 01:40:37 +0000
+++ loggerhead/changecache.py	2008-02-27 03:36:09 +0000
@@ -25,45 +25,109 @@
 cached a change, it's good forever.
 """
 
-import logging
+import cPickle
 import os
-import shelve
-import threading
 import time
 
 from loggerhead import util
-from loggerhead.util import decorator
 from loggerhead.lockfile import LockFile
 
 
 with_lock = util.with_lock('_lock', 'ChangeCache')
-        
+
+SQLITE_INTERFACE = os.environ.get('SQLITE_INTERFACE', 'sqlite')
+
+if SQLITE_INTERFACE == 'pysqlite2':
+    from pysqlite2 import dbapi2
+    _param_marker = '?'
+elif SQLITE_INTERFACE == 'sqlite':
+    import sqlite as dbapi2
+    _param_marker = '%s'
+else:
+    raise AssertionError("bad sqlite interface %r!?"%SQLITE_INTERFACE)
+
+_select_stmt = ("select data from revisiondata where revid = ?"
+                ).replace('?', _param_marker)
+_insert_stmt = ("insert into revisiondata (revid, data) "
+                "values (?, ?)").replace('?', _param_marker)
+_update_stmt = ("update revisiondata set data = ? where revid = ?"
+                ).replace('?', _param_marker)
+
+
+
+
+class FakeShelf(object):
+    def __init__(self, filename):
+        create_table = not os.path.exists(filename)
+        self.connection = dbapi2.connect(filename)
+        self.cursor = self.connection.cursor()
+        if create_table:
+            self._create_table()
+    def _create_table(self):
+        self.cursor.execute(
+            "create table RevisionData "
+            "(revid binary primary key, data binary)")
+        self.connection.commit()
+    def _serialize(self, obj):
+        r = dbapi2.Binary(cPickle.dumps(obj, protocol=2))
+        return r
+    def _unserialize(self, data):
+        return cPickle.loads(str(data))
+    def get(self, revid):
+        self.cursor.execute(_select_stmt, (revid,))
+        filechange = self.cursor.fetchone()
+        if filechange is None:
+            return None
+        else:
+            return self._unserialize(filechange[0])
+    def add(self, revid_obj_pairs, commit=True):
+        for  (r, d) in revid_obj_pairs:
+            self.cursor.execute(_insert_stmt, (r, self._serialize(d)))
+        if commit:
+            self.connection.commit()
+    def update(self, revid_obj_pairs, commit=True):
+        for  (r, d) in revid_obj_pairs:
+            self.cursor.execute(_update_stmt, (self._serialize(d), r))
+        if commit:
+            self.connection.commit()
+    def count(self):
+        self.cursor.execute(
+            "select count(*) from revisiondata")
+        return self.cursor.fetchone()[0]
+    def close(self, commit=False):
+        if commit:
+            self.connection.commit()
+        self.connection.close()
 
 class ChangeCache (object):
-    
+
     def __init__(self, history, cache_path):
         self.history = history
         self.log = history.log
-        
+
         if not os.path.exists(cache_path):
             os.mkdir(cache_path)
 
-        self._changes_filename = os.path.join(cache_path, 'changes')
-        
+        self._changes_filename = os.path.join(cache_path, 'changes.sql')
+
         # use a lockfile since the cache folder could be shared across different processes.
         self._lock = LockFile(os.path.join(cache_path, 'lock'))
         self._closed = False
-        
-        # this is fluff; don't slow down startup time with it.
-        def log_sizes():
-            self.log.info('Using change cache %s; %d entries.' % (cache_path, self.size()))
-        threading.Thread(target=log_sizes).start()
-    
+
+##         # this is fluff; don't slow down startup time with it.
+##         # but it is racy in tests :(
+##         def log_sizes():
+##             self.log.info('Using change cache %s; %d entries.' % (cache_path, self.size()))
+##         threading.Thread(target=log_sizes).start()
+
+    def _cache(self):
+        return FakeShelf(self._changes_filename)
+
     @with_lock
     def close(self):
         self.log.debug('Closing cache file.')
         self._closed = True
-    
+
     @with_lock
     def closed(self):
         return self._closed
@@ -71,7 +135,7 @@
     @with_lock
     def flush(self):
         pass
-    
+
     @with_lock
     def get_changes(self, revid_list):
         """
@@ -79,51 +143,43 @@
         from the cache are fetched by calling L{History.get_change_uncached}
         and inserted into the cache before returning.
         """
-        cache = shelve.open(self._changes_filename, 'c', protocol=2)
-
-        try:
-            out = []
-            fetch_list = []
-            sfetch_list = []
-            for revid in revid_list:
-                # if the revid is in unicode, use the utf-8 encoding as the key
-                srevid = util.to_utf8(revid)
-
-                if srevid in cache:
-                    out.append(cache[srevid])
-                else:
-                    #self.log.debug('Entry cache miss: %r' % (revid,))
-                    out.append(None)
-                    fetch_list.append(revid)
-                    sfetch_list.append(srevid)
-
-            if len(fetch_list) > 0:
-                # some revisions weren't in the cache; fetch them
-                changes = self.history.get_changes_uncached(fetch_list)
-                if len(changes) == 0:
-                    return changes
-                for i in xrange(len(revid_list)):
-                    if out[i] is None:
-                        cache[sfetch_list.pop(0)] = out[i] = changes.pop(0)
-            return out
-        finally:
-            cache.close()
-    
+        out = []
+        missing_revids = []
+        missing_revid_indices = []
+        cache = self._cache()
+        for revid in revid_list:
+            entry = cache.get(revid)
+            if entry is not None:
+                out.append(entry)
+            else:
+                missing_revids.append(revid)
+                missing_revid_indices.append(len(out))
+                out.append(None)
+        if missing_revids:
+            missing_entries = self.history.get_changes_uncached(missing_revids)
+            missing_entry_dict = {}
+            for entry in missing_entries:
+                missing_entry_dict[entry.revid] = entry
+            revid_entry_pairs = []
+            for i, revid in zip(missing_revid_indices, missing_revids):
+                out[i] = entry = missing_entry_dict.get(revid)
+                if entry is not None:
+                    revid_entry_pairs.append((revid, entry))
+            cache.add(revid_entry_pairs)
+        return filter(None, out)
+
     @with_lock
     def full(self):
-        cache = shelve.open(self._changes_filename, 'c', protocol=2)
-        try:
-            return (len(cache) >= len(self.history.get_revision_history())) and (util.to_utf8(self.history.last_revid) in cache)
-        finally:
-            cache.close()
+        cache = self._cache()
+        last_revid = util.to_utf8(self.history.last_revid)
+        revision_history = self.history.get_revision_history()
+        return (cache.count() >= len(revision_history)
+                and cache.get(last_revid) is not None)
 
     @with_lock
     def size(self):
-        cache = shelve.open(self._changes_filename, 'c', protocol=2)
-        s1 = len(cache)
-        cache.close()
-        return s1
-        
+        return self._cache().count()
+
     def check_rebuild(self, max_time=3600):
         """
         check if we need to fill in any missing pieces of the cache.  pull in
@@ -132,7 +188,7 @@
         """
         if self.closed() or self.full():
             return
-        
+
         self.log.info('Building revision cache...')
         start_time = time.time()
         last_update = time.time()
@@ -162,4 +218,39 @@
         self.log.info('Revision cache rebuild completed.')
         self.flush()
 
-
+class FileChangeCache(object):
+    def __init__(self, history, cache_path):
+        self.history = history
+
+        if not os.path.exists(cache_path):
+            os.mkdir(cache_path)
+
+        self._changes_filename = os.path.join(cache_path, 'filechanges.sql')
+
+        # use a lockfile since the cache folder could be shared across
+        # different processes.
+        self._lock = LockFile(os.path.join(cache_path, 'filechange-lock'))
+
+    @with_lock
+    def get_file_changes(self, entries):
+        out = []
+        missing_entries = []
+        missing_entry_indices = []
+        cache = FakeShelf(self._changes_filename)
+        for entry in entries:
+            changes = cache.get(entry.revid)
+            if changes is not None:
+                out.append(changes)
+            else:
+                missing_entries.append(entry)
+                missing_entry_indices.append(len(out))
+                out.append(None)
+        if missing_entries:
+            missing_changes = self.history.get_file_changes_uncached(missing_entries)
+            revid_changes_pairs = []
+            for i, entry, changes in zip(
+                missing_entry_indices, missing_entries, missing_changes):
+                revid_changes_pairs.append((entry.revid, changes))
+                out[i] = changes
+            cache.add(revid_changes_pairs)
+        return out

=== modified file 'loggerhead/controllers/__init__.py'
--- loggerhead/controllers/__init__.py	2007-06-19 00:27:53 +0000
+++ loggerhead/controllers/__init__.py	2008-02-27 03:35:24 +0000
@@ -20,17 +20,14 @@
 import logging
 import os
 import re
-import sys
-import time
 
 import turbogears
 from turbogears import controllers
-from cherrypy import NotFound
 from configobj import ConfigObj
 
 from loggerhead import util
 from loggerhead.branchview import BranchView
-from loggerhead.history import History, is_branch
+from loggerhead.history import is_branch
 
 log = logging.getLogger("loggerhead.controllers")
 

=== modified file 'loggerhead/controllers/annotate_ui.py'
--- loggerhead/controllers/annotate_ui.py	2007-07-04 14:51:40 +0000
+++ loggerhead/controllers/annotate_ui.py	2008-02-27 03:35:24 +0000
@@ -17,15 +17,13 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import datetime
 import logging
 import os
 import posixpath
-import textwrap
 import time
 
 import turbogears
-from cherrypy import HTTPError, InternalError, session
+from cherrypy import HTTPError
 
 from loggerhead import util
 
@@ -47,50 +45,53 @@
         self.log = branch.log
 
     @util.strip_whitespace
-    @turbogears.expose(html='zpt:loggerhead.templates.annotate')
-    #@turbogears.expose(html='loggerhead.templates.annotate')
+    @turbogears.expose(html='loggerhead.templates.annotate')
     def default(self, *args, **kw):
         z = time.time()
         h = self._branch.get_history()
         util.set_context(kw)
-        
-        if len(args) > 0:
-            revid = h.fix_revid(args[0])
-        else:
-            revid = h.last_revid
-
-        path = None
-        if len(args) > 1:
-            path = '/'.join(args[1:])
-            if not path.startswith('/'):
-                path = '/' + path
-        
-        file_id = kw.get('file_id', None)
-        if (file_id is None) and (path is None):
-            raise HTTPError(400, 'No file_id or filename provided to annotate')
-
-        if file_id is None:
-            file_id = h.get_file_id(revid, path)
-            
-        # no navbar for revisions
-        navigation = util.Container()
-        
-        if path is None:
-            path = h.get_path(revid, file_id)
-        filename = os.path.basename(path)
-
-        vals = {
-            'branch': self._branch,
-            'util': util,
-            'revid': revid,
-            'file_id': file_id,
-            'path': path,
-            'filename': filename,
-            'history': h,
-            'navigation': navigation,
-            'change': h.get_changes([ revid ])[0],
-            'contents': list(h.annotate_file(file_id, revid)),
-        }
-        h.flush_cache()
-        self.log.info('/annotate: %r secs' % (time.time() - z,))
-        return vals
+
+        h._branch.lock_read()
+        try:
+            if len(args) > 0:
+                revid = h.fix_revid(args[0])
+            else:
+                revid = h.last_revid
+
+            path = None
+            if len(args) > 1:
+                path = '/'.join(args[1:])
+                if not path.startswith('/'):
+                    path = '/' + path
+
+            file_id = kw.get('file_id', None)
+            if (file_id is None) and (path is None):
+                raise HTTPError(400, 'No file_id or filename provided to annotate')
+
+            if file_id is None:
+                file_id = h.get_file_id(revid, path)
+
+            # no navbar for revisions
+            navigation = util.Container()
+
+            if path is None:
+                path = h.get_path(revid, file_id)
+            filename = os.path.basename(path)
+
+            vals = {
+                'branch': self._branch,
+                'util': util,
+                'revid': revid,
+                'file_id': file_id,
+                'path': path,
+                'filename': filename,
+                'history': h,
+                'navigation': navigation,
+                'change': h.get_changes([ revid ])[0],
+                'contents': list(h.annotate_file(file_id, revid)),
+            }
+            h.flush_cache()
+            self.log.info('/annotate: %r secs' % (time.time() - z,))
+            return vals
+        finally:
+            h._branch.unlock()

=== modified file 'loggerhead/controllers/atom_ui.py'
--- loggerhead/controllers/atom_ui.py	2007-06-27 17:06:57 +0000
+++ loggerhead/controllers/atom_ui.py	2008-02-27 03:35:24 +0000
@@ -17,7 +17,6 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import cherrypy
 import turbogears
 
 from loggerhead import util
@@ -33,17 +32,22 @@
     @turbogears.expose(template='loggerhead.templates.atom', format="xml", content_type="application/atom+xml")
     def default(self, *args):
         h = self._branch.get_history()
-        pagesize = int(self._branch.config.get('pagesize', '20'))
-
-        revid_list = h.get_file_view(h.last_revid, None)
-        entries = list(h.get_changes(list(revid_list)[:pagesize]))
-
-        vals = {
-            'branch': self._branch,
-            'changes': entries,
-            'util': util,
-            'history': h,
-            'updated': entries[0].date.isoformat() + 'Z',
-        }
-        h.flush_cache()
-        return vals
+
+        h._branch.lock_read()
+        try:
+            pagesize = int(self._branch.config.get('pagesize', '20'))
+
+            revid_list = h.get_file_view(h.last_revid, None)
+            entries = list(h.get_changes(list(revid_list)[:pagesize]))
+
+            vals = {
+                'branch': self._branch,
+                'changes': entries,
+                'util': util,
+                'history': h,
+                'updated': entries[0].date.isoformat() + 'Z',
+            }
+            h.flush_cache()
+            return vals
+        finally:
+            h._branch.unlock()

=== modified file 'loggerhead/controllers/bundle_ui.py'
--- loggerhead/controllers/bundle_ui.py	2007-06-26 04:32:43 +0000
+++ loggerhead/controllers/bundle_ui.py	2008-02-27 03:35:24 +0000
@@ -16,14 +16,11 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import os
 import time
 
 import turbogears
 from cherrypy import HTTPRedirect, InternalError, response
 
-from loggerhead import util
-
 
 class BundleUI (object):
 
@@ -38,22 +35,26 @@
         z = time.time()
         h = self._branch.get_history()
 
-        if len(args) < 1:
-            raise HTTPRedirect(self._branch.url('/changes'))
-        revid = h.fix_revid(args[0])
-        if len(args) >= 3:
-            compare_revid = h.fix_revid(args[1])
-        else:
-            compare_revid = None
-        
+        h._branch.lock_read()
         try:
-            bundle_data = h.get_bundle(revid, compare_revid)
-        except:
-            self.log.exception('Exception fetching bundle')
-            raise InternalError('Could not fetch bundle')
-            
-        response.headers['Content-Type'] = 'text/plain'
-        response.headers['Content-Length'] = len(bundle_data)
-        response.body = bundle_data
-        self.log.info('/bundle: %r seconds' % (time.time() - z,))
-        return response.body
+            if len(args) < 1:
+                raise HTTPRedirect(self._branch.url('/changes'))
+            revid = h.fix_revid(args[0])
+            if len(args) >= 3:
+                compare_revid = h.fix_revid(args[1])
+            else:
+                compare_revid = None
+
+            try:
+                bundle_data = h.get_bundle(revid, compare_revid)
+            except:
+                self.log.exception('Exception fetching bundle')
+                raise InternalError('Could not fetch bundle')
+
+            response.headers['Content-Type'] = 'application/octet-stream'
+            response.headers['Content-Length'] = len(bundle_data)
+            response.body = bundle_data
+            self.log.info('/bundle: %r seconds' % (time.time() - z,))
+            return response.body
+        finally:
+            h._branch.unlock()

=== modified file 'loggerhead/controllers/changelog_ui.py'
--- loggerhead/controllers/changelog_ui.py	2007-06-27 15:38:47 +0000
+++ loggerhead/controllers/changelog_ui.py	2008-02-27 03:35:24 +0000
@@ -17,11 +17,10 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import os
 import time
 
 import turbogears
-from cherrypy import InternalError, session
+from cherrypy import InternalError
 
 from loggerhead import util
 
@@ -36,81 +35,84 @@
     @util.strip_whitespace
     @turbogears.expose(html='loggerhead.templates.changelog')
     def default(self, *args, **kw):
-        print args
         z = time.time()
         h = self._branch.get_history()
         config = self._branch.config
-        
-        if len(args) > 0:
-            revid = h.fix_revid(args[0])
-        else:
-            revid = None
 
-        file_id = kw.get('file_id', None)
-        query = kw.get('q', None)
-        start_revid = h.fix_revid(kw.get('start_revid', None))
-        orig_start_revid = start_revid
-        pagesize = int(config.get('pagesize', '20'))
-        search_failed = False
-        
+        h._branch.lock_read()
         try:
-            revid, start_revid, revid_list = h.get_view(revid, start_revid, file_id, query)
-            kw['start_revid'] = start_revid
-            util.set_context(kw)
-            
-            if (query is not None) and (len(revid_list) == 0):
-                search_failed = True
-
-            if len(revid_list) == 0:
-                scan_list = revid_list
+            if len(args) > 0:
+                revid = h.fix_revid(args[0])
             else:
-                if revid in revid_list: # XXX is this always true?
-                    i = revid_list.index(revid)
+                revid = None
+
+            file_id = kw.get('file_id', None)
+            query = kw.get('q', None)
+            start_revid = h.fix_revid(kw.get('start_revid', None))
+            orig_start_revid = start_revid
+            pagesize = int(config.get('pagesize', '20'))
+            search_failed = False
+
+            try:
+                revid, start_revid, revid_list = h.get_view(revid, start_revid, file_id, query)
+                kw['start_revid'] = start_revid
+                util.set_context(kw)
+
+                if (query is not None) and (len(revid_list) == 0):
+                    search_failed = True
+
+                if len(revid_list) == 0:
+                    scan_list = revid_list
                 else:
-                    i = None
-                scan_list = revid_list[i:]
-            entry_list = scan_list[:pagesize]
-            entries = h.get_changes(entry_list)
-        except:
-            self.log.exception('Exception fetching changes')
-            raise InternalError('Could not fetch changes')
-
-        navigation = util.Container(pagesize=pagesize, revid=revid, start_revid=start_revid, revid_list=revid_list,
-                                    file_id=file_id, scan_url='/changes', branch=self._branch, feed=True)
-        if query is not None:
-            navigation.query = query
-        util.fill_in_navigation(navigation)
-        
-        entries = list(entries)
-        # add parent & merge-point branch-nick info, in case it's useful
-        h.get_branch_nicks(entries)
-        
-        # does every change on this page have the same committer?  if so,
-        # tell the template to show committer info in the "details block"
-        # instead of on each line.
-        all_same_author = True
-
-        if entries:
-            author = entries[0].author
-            for e in entries[1:]:
-                if e.author != author:
-                    all_same_author = False
-                    break
-
-        vals = {
-            'branch': self._branch,
-            'changes': entries,
-            'util': util,
-            'history': h,
-            'revid': revid,
-            'navigation': navigation,
-            'file_id': file_id,
-            'start_revid': start_revid,
-            'viewing_from': (orig_start_revid is not None) and (orig_start_revid != h.last_revid),
-            'query': query,
-            'search_failed': search_failed,
-            'all_same_author': all_same_author,
-        }
-        h.flush_cache()
-        self.log.info('/changes %r: %r secs' % (revid, time.time() - z))
-        return vals
+                    if revid in revid_list: # XXX is this always true?
+                        i = revid_list.index(revid)
+                    else:
+                        i = None
+                    scan_list = revid_list[i:]
+                change_list = scan_list[:pagesize]
+                changes = list(h.get_changes(change_list))
+                h.add_changes(changes)
+            except:
+                self.log.exception('Exception fetching changes')
+                raise InternalError('Could not fetch changes')
+
+            navigation = util.Container(pagesize=pagesize, revid=revid, start_revid=start_revid, revid_list=revid_list,
+                                        file_id=file_id, scan_url='/changes', branch=self._branch, feed=True)
+            if query is not None:
+                navigation.query = query
+            util.fill_in_navigation(navigation)
+
+            # add parent & merge-point branch-nick info, in case it's useful
+            h.get_branch_nicks(changes)
+
+            # does every change on this page have the same committer?  if so,
+            # tell the template to show committer info in the "details block"
+            # instead of on each line.
+            all_same_author = True
+
+            if changes:
+                author = changes[0].author
+                for e in changes[1:]:
+                    if e.author != author:
+                        all_same_author = False
+                        break
+
+            vals = {
+                'branch': self._branch,
+                'changes': changes,
+                'util': util,
+                'history': h,
+                'revid': revid,
+                'navigation': navigation,
+                'file_id': file_id,
+                'start_revid': start_revid,
+                'viewing_from': (orig_start_revid is not None) and (orig_start_revid != h.last_revid),
+                'query': query,
+                'search_failed': search_failed,
+                'all_same_author': all_same_author,
+            }
+            h.flush_cache()
+            self.log.info('/changes %r: %r secs' % (revid, time.time() - z))
+            return vals
+        finally:
+            h._branch.unlock()

=== modified file 'loggerhead/controllers/download_ui.py'
--- loggerhead/controllers/download_ui.py	2007-05-23 17:42:19 +0000
+++ loggerhead/controllers/download_ui.py	2008-02-27 03:35:24 +0000
@@ -17,18 +17,12 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import datetime
 import logging
 import mimetypes
-import os
-import posixpath
-import textwrap
 import time
 
 import turbogears
-from cherrypy import HTTPRedirect, response, session
-
-from loggerhead import util
+from cherrypy import HTTPRedirect, response
 
 
 log = logging.getLogger("loggerhead.controllers")
@@ -47,19 +41,23 @@
         z = time.time()
         h = self._branch.get_history()
         
-        if len(args) < 2:
-            raise HTTPRedirect(self._branch.url('/changes'))
-        
-        revid = h.fix_revid(args[0])
-        file_id = args[1]
-        path, filename, content = h.get_file(file_id, revid)
-        mime_type, encoding = mimetypes.guess_type(filename)
-        if mime_type is None:
-            mime_type = 'application/octet-stream'
-
-        self.log.info('/download %s @ %s (%d bytes)', path, h.get_revno(revid), len(content))
-        response.headers['Content-Type'] = mime_type
-        response.headers['Content-Length'] = len(content)
-        response.headers['Content-Disposition'] = 'attachment; filename=%s'%(filename,)
-        response.body = content
-        return response.body
+        h._branch.lock_read()
+        try:
+            if len(args) < 2:
+                raise HTTPRedirect(self._branch.url('/changes'))
+
+            revid = h.fix_revid(args[0])
+            file_id = args[1]
+            path, filename, content = h.get_file(file_id, revid)
+            mime_type, encoding = mimetypes.guess_type(filename)
+            if mime_type is None:
+                mime_type = 'application/octet-stream'
+
+            self.log.info('/download %s @ %s (%d bytes)', path, h.get_revno(revid), len(content))
+            response.headers['Content-Type'] = mime_type
+            response.headers['Content-Length'] = len(content)
+            response.headers['Content-Disposition'] = 'attachment; filename=%s'%(filename,)
+            response.body = content
+            return response.body
+        finally:
+            h._branch.unlock()

=== modified file 'loggerhead/controllers/inventory_ui.py'
--- loggerhead/controllers/inventory_ui.py	2007-06-27 16:52:56 +0000
+++ loggerhead/controllers/inventory_ui.py	2008-02-27 03:35:24 +0000
@@ -17,15 +17,12 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import datetime
 import logging
-import os
 import posixpath
-import textwrap
 import time
 
 import turbogears
-from cherrypy import InternalError, session
+from cherrypy import InternalError
 
 from loggerhead import util
 
@@ -52,63 +49,59 @@
         z = time.time()
         h = self._branch.get_history()
         util.set_context(kw)
-        
-        if len(args) > 0:
-            revid = h.fix_revid(args[0])
-        else:
-            revid = h.last_revid
-        
-        try:
-            rev = h.get_revision(revid)
-            inv = h.get_inventory(revid)
-        except:
-            self.log.exception('Exception fetching changes')
-            raise InternalError('Could not fetch changes')
-
-        file_id = kw.get('file_id', inv.root.file_id)
-        sort_type = kw.get('sort', None)
-
-        try:
-            rev = h.get_revision(revid)
-            inv = h.get_inventory(revid)
-        except:
-            self.log.exception('Exception fetching changes')
-            raise InternalError('Could not fetch changes')
-
-        # no navbar for revisions
-        navigation = util.Container()
-
-        change = h.get_changes([ revid ])[0]
-        # add parent & merge-point branch-nick info, in case it's useful
-        h.get_branch_nicks([ change ])
-        
-        path = inv.id2path(file_id)
-        if not path.startswith('/'):
-            path = '/' + path
-        idpath = inv.get_idpath(file_id)
-        if len(idpath) > 1:
-            updir = dirname(path)
-            updir_file_id = idpath[-2]
-        else:
-            updir = None
-            updir_file_id = None
-        if updir == '/':
-            updir_file_id = None
-
-        vals = {
-            'branch': self._branch,
-            'util': util,
-            'revid': revid,
-            'change': change,
-            'file_id': file_id,
-            'path': path,
-            'updir': updir,
-            'updir_file_id': updir_file_id,
-            'filelist': h.get_filelist(inv, file_id, sort_type),
-            'history': h,
-            'posixpath': posixpath,
-            'navigation': navigation,
-        }
-        h.flush_cache()
-        self.log.info('/inventory %r: %r secs' % (revid, time.time() - z))
-        return vals
+
+        h._branch.lock_read()
+        try:
+            if len(args) > 0:
+                revid = h.fix_revid(args[0])
+            else:
+                revid = h.last_revid
+
+            try:
+                inv = h.get_inventory(revid)
+            except:
+                self.log.exception('Exception fetching changes')
+                raise InternalError('Could not fetch changes')
+
+            file_id = kw.get('file_id', inv.root.file_id)
+            sort_type = kw.get('sort', None)
+
+            # no navbar for revisions
+            navigation = util.Container()
+
+            change = h.get_changes([ revid ])[0]
+            # add parent & merge-point branch-nick info, in case it's useful
+            h.get_branch_nicks([ change ])
+
+            path = inv.id2path(file_id)
+            if not path.startswith('/'):
+                path = '/' + path
+            idpath = inv.get_idpath(file_id)
+            if len(idpath) > 1:
+                updir = dirname(path)
+                updir_file_id = idpath[-2]
+            else:
+                updir = None
+                updir_file_id = None
+            if updir == '/':
+                updir_file_id = None
+
+            vals = {
+                'branch': self._branch,
+                'util': util,
+                'revid': revid,
+                'change': change,
+                'file_id': file_id,
+                'path': path,
+                'updir': updir,
+                'updir_file_id': updir_file_id,
+                'filelist': h.get_filelist(inv, file_id, sort_type),
+                'history': h,
+                'posixpath': posixpath,
+                'navigation': navigation,
+            }
+            h.flush_cache()
+            self.log.info('/inventory %r: %r secs' % (revid, time.time() - z))
+            return vals
+        finally:
+            h._branch.unlock()

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2007-06-12 15:57:50 +0000
+++ loggerhead/controllers/revision_ui.py	2008-02-27 03:35:24 +0000
@@ -17,14 +17,10 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import datetime
-import logging
-import os
-import textwrap
 import time
 
 import turbogears
-from cherrypy import InternalError, session
+from cherrypy import InternalError
 
 from loggerhead import util
 
@@ -44,55 +40,56 @@
         h = self._branch.get_history()
         util.set_context(kw)
         
-        if len(args) > 0:
-            revid = h.fix_revid(args[0])
-        else:
-            revid = None
-        
-        file_id = kw.get('file_id', None)
-        start_revid = h.fix_revid(kw.get('start_revid', None))
-        query = kw.get('q', None)
-        remember = kw.get('remember', None)
-        compare_revid = kw.get('compare_revid', None)
-        
+        h._branch.lock_read()
         try:
-            revid, start_revid, revid_list = h.get_view(revid, start_revid, file_id, query)
-        except:
-            self.log.exception('Exception fetching changes')
-            raise InternalError('Could not fetch changes')
-        
-        navigation = util.Container(revid_list=revid_list, revid=revid, start_revid=start_revid, file_id=file_id,
-                                    pagesize=1, scan_url='/revision', branch=self._branch, feed=True)
-        if query is not None:
-            navigation.query = query
-        util.fill_in_navigation(navigation)
-
-        if compare_revid is not None:
-            change = h.get_diff(compare_revid, revid)
-        else:
-            change = h.get_changes([ revid ], get_diffs=True)[0]
-        # add parent & merge-point branch-nick info, in case it's useful
-        h.get_branch_nicks([ change ])
-
-        # let's make side-by-side diff be the default
-        side_by_side = not kw.get('unified', False)
-        if side_by_side:
-            h.add_side_by_side([ change ])
-        
-        vals = {
-            'branch': self._branch,
-            'revid': revid,
-            'change': change,
-            'start_revid': start_revid,
-            'file_id': file_id,
-            'util': util,
-            'history': h,
-            'navigation': navigation,
-            'query': query,
-            'remember': remember,
-            'compare_revid': compare_revid,
-            'side_by_side': side_by_side,
-        }
-        h.flush_cache()
-        self.log.info('/revision: %r seconds' % (time.time() - z,))
-        return vals
+            if len(args) > 0:
+                revid = h.fix_revid(args[0])
+            else:
+                revid = None
+
+            file_id = kw.get('file_id', None)
+            start_revid = h.fix_revid(kw.get('start_revid', None))
+            query = kw.get('q', None)
+            remember = kw.get('remember', None)
+            compare_revid = kw.get('compare_revid', None)
+
+            try:
+                revid, start_revid, revid_list = h.get_view(revid, start_revid, file_id, query)
+            except:
+                self.log.exception('Exception fetching changes')
+                raise InternalError('Could not fetch changes')
+
+            navigation = util.Container(revid_list=revid_list, revid=revid, start_revid=start_revid, file_id=file_id,
+                                        pagesize=1, scan_url='/revision', branch=self._branch, feed=True)
+            if query is not None:
+                navigation.query = query
+            util.fill_in_navigation(navigation)
+
+            change = h.get_change_with_diff(revid, compare_revid)
+            # add parent & merge-point branch-nick info, in case it's useful
+            h.get_branch_nicks([ change ])
+
+            # let's make side-by-side diff be the default
+            side_by_side = not kw.get('unified', False)
+            if side_by_side:
+                h.add_side_by_side([ change ])
+
+            vals = {
+                'branch': self._branch,
+                'revid': revid,
+                'change': change,
+                'start_revid': start_revid,
+                'file_id': file_id,
+                'util': util,
+                'history': h,
+                'navigation': navigation,
+                'query': query,
+                'remember': remember,
+                'compare_revid': compare_revid,
+                'side_by_side': side_by_side,
+            }
+            h.flush_cache()
+            self.log.info('/revision: %r seconds' % (time.time() - z,))
+            return vals
+        finally:
+            h._branch.unlock()

=== modified file 'loggerhead/daemon.py'
--- loggerhead/daemon.py	2007-06-18 23:07:53 +0000
+++ loggerhead/daemon.py	2008-02-27 03:36:09 +0000
@@ -1,7 +1,6 @@
 # daemon code from ASPN
 
 import os
-import sys
 
 
 def daemonize(pidfile, home):
@@ -20,41 +19,41 @@
     except OSError, e:
         raise Exception("%s [%d]" % (e.strerror, e.errno))
 
-    if (pid == 0):	# The first child.
+    if (pid == 0):      # The first child.
         os.setsid()
 
         try:
-            pid = os.fork()	# Fork a second child.
+            pid = os.fork()     # Fork a second child.
         except OSError, e:
             raise Exception, "%s [%d]" % (e.strerror, e.errno)
 
-        if (pid == 0):	# The second child.
+        if (pid == 0):  # The second child.
             os.chdir(WORKDIR)
             os.umask(UMASK)
         else:
-            os._exit(0)	# Exit parent (the first child) of the second child.
+            os._exit(0) # Exit parent (the first child) of the second child.
     else:
-        os._exit(0)	# Exit parent of the first child.
+        os._exit(0)     # Exit parent of the first child.
 
-    #import resource		# Resource usage information.
+    #import resource            # Resource usage information.
     #maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
     #if (maxfd == resource.RLIM_INFINITY):
     #    maxfd = MAXFD
-  
+
     # Iterate through and close all file descriptors.
     #for fd in range(3, maxfd):
     #    try:
     #        os.close(fd)
-    #    except OSError:	# ERROR, fd wasn't open to begin with (ignored)
+    #    except OSError:        # ERROR, fd wasn't open to begin with (ignored)
     #        pass
 
     # This call to open is guaranteed to return the lowest file descriptor,
     # which will be 0 (stdin), since it was closed above.
-    #os.open(REDIRECT_TO, os.O_RDWR)	# standard input (0)
+    #os.open(REDIRECT_TO, os.O_RDWR)    # standard input (0)
 
     # Duplicate standard input to standard output and standard error.
-    #os.dup2(0, 1)			# standard output (1)
-    #os.dup2(0, 2)			# standard error (2)
+    #os.dup2(0, 1)                      # standard output (1)
+    #os.dup2(0, 2)                      # standard error (2)
 
     f = open(pidfile, 'w')
     f.write('%d\n' % (os.getpid(),))

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2007-06-27 17:06:57 +0000
+++ loggerhead/history.py	2008-02-27 03:36:09 +0000
@@ -27,14 +27,9 @@
 
 
 import bisect
-import cgi
 import datetime
 import logging
-import os
-import posixpath
 import re
-import shelve
-import sys
 import textwrap
 import threading
 import time
@@ -44,20 +39,19 @@
 from loggerhead.util import decorator
 
 import bzrlib
-import bzrlib.annotate
 import bzrlib.branch
 import bzrlib.bundle.serializer
-import bzrlib.decorators
 import bzrlib.diff
 import bzrlib.errors
 import bzrlib.progress
-import bzrlib.textfile
+import bzrlib.revision
 import bzrlib.tsort
 import bzrlib.ui
 
 
 with_branch_lock = util.with_lock('_lock', 'branch')
 
+
 @decorator
 def with_bzrlib_read_lock(unbound):
     def bzrlib_read_locked(self, *args, **kw):
@@ -100,7 +94,7 @@
     """
     turn a normal unified-style diff (post-processed by parse_delta) into a
     side-by-side diff structure.  the new structure is::
-    
+
         chunks: list(
             diff: list(
                 old_lineno: int,
@@ -142,20 +136,28 @@
 
 
 def clean_message(message):
-    # clean up a commit message and return it and a short (1-line) version
+    """Clean up a commit message and return it and a short (1-line) version.
+
+    Commit messages that are long single lines are reflowed using the textwrap
+    module (Robey, the original author of this code, apparently favored this
+    style of message).
+    """
     message = message.splitlines()
+
     if len(message) == 1:
-        # robey-style 1-line long message
         message = textwrap.wrap(message[0])
-    elif len(message) == 0:
-        # sometimes a commit may have NO message!
-        message = ['']
-        
-    # make short form of commit message
+
+    if len(message) == 0:
+        # We can end up where when (a) the commit message was empty or (b)
+        # when the message consisted entirely of whitespace, in which case
+        # textwrap.wrap() returns an empty list.
+        return [''], ''
+
+    # Make short form of commit message.
     short_message = message[0]
     if len(short_message) > 80:
         short_message = short_message[:80] + '...'
-    
+
     return message, short_message
 
 
@@ -165,7 +167,7 @@
     if kind == 'symlink':
         path += '@'
     return path
-        
+
 
 
 # from bzrlib
@@ -187,68 +189,84 @@
 
 
 class History (object):
-    
+
     def __init__(self):
         self._change_cache = None
+        self._file_change_cache = None
         self._index = None
         self._lock = threading.RLock()
-    
+
     @classmethod
     def from_branch(cls, branch, name=None):
         z = time.time()
         self = cls()
         self._branch = branch
         self._last_revid = self._branch.last_revision()
-        if self._last_revid is not None:
-            self._revision_graph = branch.repository.get_revision_graph(self._last_revid)
-        else:
-            self._revision_graph = {}
-        
+        self._revision_graph = branch.repository.get_revision_graph(self._last_revid)
+
         if name is None:
             name = self._branch.nick
         self._name = name
         self.log = logging.getLogger('loggerhead.%s' % (name,))
-        
+
         self._full_history = []
         self._revision_info = {}
         self._revno_revid = {}
-        self._merge_sort = bzrlib.tsort.merge_sort(self._revision_graph, self._last_revid, generate_revno=True)
+        if bzrlib.revision.is_null(self._last_revid):
+            self._merge_sort = []
+        else:
+            self._merge_sort = bzrlib.tsort.merge_sort(
+                self._revision_graph, self._last_revid, generate_revno=True)
         for (seq, revid, merge_depth, revno, end_of_merge) in self._merge_sort:
             self._full_history.append(revid)
             revno_str = '.'.join(str(n) for n in revno)
             self._revno_revid[revno_str] = revid
             self._revision_info[revid] = (seq, revid, merge_depth, revno_str, end_of_merge)
-
         # cache merge info
         self._where_merged = {}
         for revid in self._revision_graph.keys():
-            if not revid in self._full_history: 
+            if self._revision_info[revid][2] == 0:
                 continue
             for parent in self._revision_graph[revid]:
                 self._where_merged.setdefault(parent, set()).add(revid)
 
         self.log.info('built revision graph cache: %r secs' % (time.time() - z,))
         return self
-    
+
     @classmethod
     def from_folder(cls, path, name=None):
         b = bzrlib.branch.Branch.open(path)
-        return cls.from_branch(b, name)
+        b.lock_read()
+        try:
+            return cls.from_branch(b, name)
+        finally:
+            b.unlock()
 
     @with_branch_lock
     def out_of_date(self):
         # the branch may have been upgraded on disk, in which case we're stale.
+        newly_opened = bzrlib.branch.Branch.open(self._branch.base)
         if self._branch.__class__ is not \
-               bzrlib.branch.Branch.open(self._branch.base).__class__:
+               newly_opened.__class__:
+            return True
+        if self._branch.repository.__class__ is not \
+               newly_opened.repository.__class__:
             return True
         return self._branch.last_revision() != self._last_revid
 
     def use_cache(self, cache):
         self._change_cache = cache
-    
+
+    def use_file_cache(self, cache):
+        self._file_change_cache = cache
+
     def use_search_index(self, index):
         self._index = index
 
+    @property
+    def has_revisions(self):
+        return not bzrlib.revision.is_null(self.last_revid)
+
     @with_branch_lock
     def detach(self):
         # called when a new history object needs to be created, because the
@@ -267,23 +285,19 @@
         if self._change_cache is None:
             return
         self._change_cache.flush()
-    
+
     def check_rebuild(self):
         if self._change_cache is not None:
             self._change_cache.check_rebuild()
         if self._index is not None:
             self._index.check_rebuild()
-    
+
     last_revid = property(lambda self: self._last_revid, None, None)
-    
+
     @with_branch_lock
     def get_config(self):
         return self._branch.get_config()
-    
-    @with_branch_lock
-    def get_revision(self, revid):
-        return self._branch.repository.get_revision(revid)
-    
+
     def get_revno(self, revid):
         if revid not in self._revision_info:
             # ghost parent?
@@ -293,7 +307,7 @@
 
     def get_revision_history(self):
         return self._full_history
-    
+
     def get_revids_from(self, revid_list, revid):
         """
         given a list of revision ids, yield revisions in graph order,
@@ -309,7 +323,7 @@
             if len(parents) == 0:
                 return
             revid = parents[0]
-    
+
     @with_branch_lock
     def get_short_revision_history_by_fileid(self, file_id):
         # wow.  is this really the only way we can get this list?  by
@@ -334,7 +348,7 @@
         revid_list.reverse()
         index = -index
         return revid_list[index:]
-    
+
     @with_branch_lock
     def get_revision_history_matching(self, revid_list, text):
         self.log.debug('searching %d revisions for %r', len(revid_list), text)
@@ -359,12 +373,12 @@
         # put them in some coherent order :)
         out = [r for r in self._full_history if r in out]
         return out
-    
+
     @with_branch_lock
     def get_search_revid_list(self, query, revid_list):
         """
         given a "quick-search" query, try a few obvious possible meanings:
-        
+
             - revision id or # ("128.1.3")
             - date (US style "mm/dd/yy", earth style "dd-mm-yy", or iso style "yyyy-mm-dd")
             - comment text as a fallback
@@ -374,14 +388,16 @@
         # FIXME: there is some silliness in this action.  we have to look up
         # all the relevant changes (time-consuming) only to return a list of
         # revids which will be used to fetch a set of changes again.
-        
+
         # if they entered a revid, just jump straight there; ignore the passed-in revid_list
         revid = self.fix_revid(query)
         if revid is not None:
+            if isinstance(revid, unicode):
+                revid = revid.encode('utf-8')
             changes = self.get_changes([ revid ])
             if (changes is not None) and (len(changes) > 0):
                 return [ revid ]
-        
+
         date = None
         m = self.us_date_re.match(query)
         if m is not None:
@@ -399,12 +415,12 @@
                 # if no limit to the query was given, search only the direct-parent path.
                 revid_list = list(self.get_revids_from(None, self._last_revid))
             return self.get_revision_history_since(revid_list, date)
-        
+
         # check comment fields.
         if revid_list is None:
             revid_list = self._full_history
         return self.get_revision_history_matching_indexed(revid_list, query)
-    
+
     revno_re = re.compile(r'^[\d\.]+$')
     # the date regex are without a final '$' so that queries like
     # "2006-11-30 12:15" still mostly work.  (i think it's better to give
@@ -422,14 +438,14 @@
         if self.revno_re.match(revid):
             revid = self._revno_revid[revid]
         return revid
-    
+
     @with_branch_lock
     def get_file_view(self, revid, file_id):
         """
         Given a revid and optional path, return a (revlist, revid) for
         navigation through the current scope: from the revid (or the latest
         revision) back to the original revision.
-        
+
         If file_id is None, the entire revision history is the list scope.
         """
         if revid is None:
@@ -442,27 +458,29 @@
         else:
             revlist = list(self.get_revids_from(None, revid))
         return revlist
-    
+
     @with_branch_lock
     def get_view(self, revid, start_revid, file_id, query=None):
         """
         use the URL parameters (revid, start_revid, file_id, and query) to
         determine the revision list we're viewing (start_revid, file_id, query)
         and where we are in it (revid).
-        
-        if a query is given, we're viewing query results.
-        if a file_id is given, we're viewing revisions for a specific file.
-        if a start_revid is given, we're viewing the branch from a
-            specific revision up the tree.
-        (these may be combined to view revisions for a specific file, from
-            a specific revision, with a specific search query.)
-            
-        returns a new (revid, start_revid, revid_list, scan_list) where:
-        
+
+            - if a query is given, we're viewing query results.
+            - if a file_id is given, we're viewing revisions for a specific
+              file.
+            - if a start_revid is given, we're viewing the branch from a
+              specific revision up the tree.
+
+        these may be combined to view revisions for a specific file, from
+        a specific revision, with a specific search query.
+
+        returns a new (revid, start_revid, revid_list) where:
+
             - revid: current position within the view
             - start_revid: starting revision of this view
             - revid_list: list of revision ids for this view
-        
+
         file_id and query are never changed so aren't returned, but they may
         contain vital context for future url navigation.
         """
@@ -479,7 +497,7 @@
                 revid_list= self.get_file_view(revid, file_id)
                 start_revid = revid
             return revid, start_revid, revid_list
-        
+
         # potentially limit the search
         if file_id is not None:
             revid_list = self.get_file_view(start_revid, file_id)
@@ -507,13 +525,13 @@
         if (len(path) > 0) and not path.startswith('/'):
             path = '/' + path
         return path
-    
+
     @with_branch_lock
     def get_file_id(self, revid, path):
         if (len(path) > 0) and not path.startswith('/'):
             path = '/' + path
         return self._branch.repository.get_revision_inventory(revid).path2id(path)
-    
+
 
     def get_merge_point_list(self, revid):
         """
@@ -521,7 +539,7 @@
         """
         if '.' not in self.get_revno(revid):
             return []
-        
+
         merge_point = []
         while True:
             children = self._where_merged.get(revid, [])
@@ -544,7 +562,7 @@
                 merge_point.extend(merge_point_next)
 
             revid = nexts[0]
-            
+
     def simplify_merge_point_list(self, revids):
         """if a revision is already merged, don't show further merge points"""
         d = {}
@@ -587,78 +605,98 @@
                     p.branch_nick = p_change_dict[p.revid].branch_nick
                 else:
                     p.branch_nick = '(missing)'
-    
+
     @with_branch_lock
-    def get_changes(self, revid_list, get_diffs=False):
+    def get_changes(self, revid_list):
+        """Return a list of changes objects for the given revids.
+
+        Revisions not present and NULL_REVISION will be ignored.
+        """
         if self._change_cache is None:
             changes = self.get_changes_uncached(revid_list)
         else:
             changes = self._change_cache.get_changes(revid_list)
         if len(changes) == 0:
             return changes
-        
+
         # some data needs to be recalculated each time, because it may
         # change as new revisions are added.
         for change in changes:
             merge_revids = self.simplify_merge_point_list(self.get_merge_point_list(change.revid))
             change.merge_points = [util.Container(revid=r, revno=self.get_revno(r)) for r in merge_revids]
+            if len(change.parents) > 0:
+                if isinstance(change.parents[0], util.Container):
+                    # old cache stored a potentially-bogus revno
+                    change.parents = [util.Container(revid=p.revid, revno=self.get_revno(p.revid)) for p in change.parents]
+                else:
+                    change.parents = [util.Container(revid=r, revno=self.get_revno(r)) for r in change.parents]
             change.revno = self.get_revno(change.revid)
-            if get_diffs:
-                # this may be time-consuming, but it only happens on the revision page, and only for one revision at a time.
-                if len(change.parents) > 0:
-                    parent_revid = change.parents[0].revid
-                else:
-                    parent_revid = None
-                change.changes.modified = self._parse_diffs(parent_revid, change.revid)
 
         parity = 0
         for change in changes:
             change.parity = parity
             parity ^= 1
-        
+
         return changes
 
-    # alright, let's profile this sucka.
-    def _get_changes_profiled(self, revid_list, get_diffs=False):
+    # alright, let's profile this sucka. (FIXME remove this eventually...)
+    def _get_changes_profiled(self, revid_list):
         from loggerhead.lsprof import profile
         import cPickle
-        ret, stats = profile(self.get_changes_uncached, revid_list, get_diffs)
+        ret, stats = profile(self.get_changes_uncached, revid_list)
         stats.sort()
         stats.freeze()
         cPickle.dump(stats, open('lsprof.stats', 'w'), 2)
         self.log.info('lsprof complete!')
         return ret
 
+    @with_branch_lock
+    @with_bzrlib_read_lock
+    def get_changes_uncached(self, revid_list):
+        revid_list = filter(lambda revid: not bzrlib.revision.is_null(revid),
+                            revid_list)
+        repo = self._branch.repository
+        rev_list = repo.get_revisions(
+            repo.get_graph().get_parent_map(revid_list))
+
+        return [self._change_from_revision(rev) for rev in rev_list]
+
     def _get_deltas_for_revisions_with_trees(self, revisions):
-        """Produce a generator of revision deltas.
-        
+        """Produce a list of revision deltas.
+
         Note that the input is a sequence of REVISIONS, not revision_ids.
         Trees will be held in memory until the generator exits.
         Each delta is relative to the revision's lefthand predecessor.
+        (This is copied from bzrlib.)
         """
         required_trees = set()
         for revision in revisions:
-            required_trees.add(revision.revision_id)
-            required_trees.update(revision.parent_ids[:1])
-        trees = dict((t.get_revision_id(), t) for 
+            required_trees.add(revision.revid)
+            required_trees.update([p.revid for p in revision.parents[:1]])
+        trees = dict((t.get_revision_id(), t) for
                      t in self._branch.repository.revision_trees(required_trees))
         ret = []
         self._branch.repository.lock_read()
         try:
             for revision in revisions:
-                if not revision.parent_ids:
-                    old_tree = self._branch.repository.revision_tree(None)
+                if not revision.parents:
+                    old_tree = self._branch.repository.revision_tree(
+                        bzrlib.revision.NULL_REVISION)
                 else:
-                    old_tree = trees[revision.parent_ids[0]]
-                tree = trees[revision.revision_id]
-                ret.append((tree, old_tree, tree.changes_from(old_tree)))
+                    old_tree = trees[revision.parents[0].revid]
+                tree = trees[revision.revid]
+                ret.append(tree.changes_from(old_tree))
             return ret
         finally:
             self._branch.repository.unlock()
-    
-    def entry_from_revision(self, revision):
+
+    def _change_from_revision(self, revision):
+        """
+        Given a bzrlib Revision, return a processed "change" for use in
+        templates.
+        """
         commit_time = datetime.datetime.fromtimestamp(revision.timestamp)
-        
+
         parents = [util.Container(revid=r, revno=self.get_revno(r)) for r in revision.parent_ids]
 
         message, short_message = clean_message(revision.message)
@@ -671,50 +709,47 @@
             'short_comment': short_message,
             'comment': revision.message,
             'comment_clean': [util.html_clean(s) for s in message],
-            'parents': parents,
+            'parents': revision.parent_ids,
         }
         return util.Container(entry)
 
-    @with_branch_lock
-    @with_bzrlib_read_lock
-    def get_changes_uncached(self, revid_list):
-        done = False
-        while not done:
-            try:
-                rev_list = self._branch.repository.get_revisions(revid_list)
-                done = True
-            except (KeyError, bzrlib.errors.NoSuchRevision), e:
-                # this sometimes happens with arch-converted branches.
-                # i don't know why. :(
-                self.log.debug('No such revision (skipping): %s', e)
-                revid_list.remove(e.revision)
-        if not revid_list:
-            return []
-        
-        delta_list = self._get_deltas_for_revisions_with_trees(rev_list)
-        combined_list = zip(rev_list, delta_list)
-        
-        entries = []
-        for rev, (new_tree, old_tree, delta) in combined_list:
-            entry = self.entry_from_revision(rev)
-            entry.changes = self.parse_delta(delta, old_tree, new_tree)
-            entries.append(entry)
-        
-        return entries
-
-    @with_bzrlib_read_lock
-    def _get_diff(self, revid1, revid2):
-        rev_tree1 = self._branch.repository.revision_tree(revid1)
-        rev_tree2 = self._branch.repository.revision_tree(revid2)
+    def get_file_changes_uncached(self, entries):
+        delta_list = self._get_deltas_for_revisions_with_trees(entries)
+
+        return [self.parse_delta(delta) for delta in delta_list]
+
+    @with_branch_lock
+    def get_file_changes(self, entries):
+        if self._file_change_cache is None:
+            return self.get_file_changes_uncached(entries)
+        else:
+            return self._file_change_cache.get_file_changes(entries)
+
+    def add_changes(self, entries):
+        changes_list = self.get_file_changes(entries)
+
+        for entry, changes in zip(entries, changes_list):
+            entry.changes = changes
+
+    @with_branch_lock
+    def get_change_with_diff(self, revid, compare_revid=None):
+        change = self.get_changes([revid])[0]
+
+        if compare_revid is None:
+            if change.parents:
+                compare_revid = change.parents[0].revid
+            else:
+                compare_revid = 'null:'
+
+        rev_tree1 = self._branch.repository.revision_tree(compare_revid)
+        rev_tree2 = self._branch.repository.revision_tree(revid)
         delta = rev_tree2.changes_from(rev_tree1)
-        return rev_tree1, rev_tree2, delta
-    
-    def get_diff(self, revid1, revid2):
-        rev_tree1, rev_tree2, delta = self._get_diff(revid1, revid2)
-        entry = self.get_changes([ revid2 ], False)[0]
-        entry.changes = self.parse_delta(delta, True, rev_tree1, rev_tree2)
-        return entry
-    
+
+        change.changes = self.parse_delta(delta)
+        change.changes.modified = self._parse_diffs(rev_tree1, rev_tree2, delta)
+
+        return change
+
     @with_branch_lock
     def get_file(self, file_id, revid):
         "returns (path, filename, data)"
@@ -725,11 +760,11 @@
         if not path.startswith('/'):
             path = '/' + path
         return path, inv_entry.name, rev_tree.get_file_text(file_id)
-    
-    def _parse_diffs(self, revid1, revid2):
+
+    def _parse_diffs(self, old_tree, new_tree, delta):
         """
         Return a list of processed diffs, in the format::
-        
+
             list(
                 filename: str,
                 file_id: str,
@@ -743,16 +778,15 @@
                 ),
             )
         """
-        old_tree, new_tree, delta = self._get_diff(revid1, revid2)
         process = []
         out = []
-        
+
         for old_path, new_path, fid, kind, text_modified, meta_modified in delta.renamed:
             if text_modified:
                 process.append((old_path, new_path, fid, kind))
         for path, fid, kind, text_modified, meta_modified in delta.modified:
             process.append((path, path, fid, kind))
-        
+
         for old_path, new_path, fid, kind in process:
             old_lines = old_tree.get_file_lines(fid)
             new_lines = new_tree.get_file_lines(fid)
@@ -768,7 +802,7 @@
             else:
                 diff = ''
             out.append(util.Container(filename=rich_filename(new_path, kind), file_id=fid, chunks=self._process_diff(diff)))
-        
+
         return out
 
     def _process_diff(self, diff):
@@ -808,12 +842,11 @@
         if chunk is not None:
             chunks.append(chunk)
         return chunks
-                
-    @with_branch_lock
-    def parse_delta(self, delta, get_diffs=True, old_tree=None, new_tree=None):
+
+    def parse_delta(self, delta):
         """
         Return a nested data structure containing the changes in a delta::
-        
+
             added: list((filename, file_id)),
             renamed: list((old_filename, new_filename, file_id)),
             deleted: list((filename, file_id)),
@@ -826,21 +859,21 @@
         modified = []
         renamed = []
         removed = []
-        
+
         for path, fid, kind in delta.added:
             added.append((rich_filename(path, kind), fid))
-        
+
         for path, fid, kind, text_modified, meta_modified in delta.modified:
             modified.append(util.Container(filename=rich_filename(path, kind), file_id=fid))
-        
+
         for old_path, new_path, fid, kind, text_modified, meta_modified in delta.renamed:
             renamed.append((rich_filename(old_path, kind), rich_filename(new_path, kind), fid))
             if meta_modified or text_modified:
                 modified.append(util.Container(filename=rich_filename(new_path, kind), file_id=fid))
-        
+
         for path, fid, kind in delta.removed:
             removed.append((rich_filename(path, kind), fid))
-        
+
         return util.Container(added=added, renamed=renamed, removed=removed, modified=modified)
 
     @staticmethod
@@ -849,7 +882,7 @@
         for change in changes:
             for m in change.changes.modified:
                 m.sbs_chunks = _make_side_by_side(m.chunks)
-    
+
     @with_branch_lock
     def get_filelist(self, inv, file_id, sort_type=None):
         """
@@ -861,25 +894,26 @@
         path = inv.id2path(file_id)
         file_list = []
 
+        revid_set = set()
+
+        for filename, entry in dir_ie.children.iteritems():
+            revid_set.add(entry.revision)
+
+        change_dict = {}
+        for change in self.get_changes(list(revid_set)):
+            change_dict[change.revid] = change
+
         for filename, entry in dir_ie.children.iteritems():
             pathname = filename
             if entry.kind == 'directory':
                 pathname += '/'
 
             revid = entry.revision
-            if self._change_cache:
-                timestamp = self.get_changes([revid])[0].date
-            else:
-                revision = self._branch.repository.get_revision(revid)
-                timestamp = datetime.datetime.fromtimestamp(revision.timestamp)
-
-            change = util.Container(date=timestamp,
-                                    revno=self.get_revno(revid))
 
             file = util.Container(
                 filename=filename, executable=entry.executable, kind=entry.kind,
                 pathname=pathname, file_id=entry.file_id, size=entry.text_size,
-                revid=revid, change=change)
+                revid=revid, change=change_dict[revid])
             file_list.append(file)
 
         if sort_type == 'filename' or sort_type is None:
@@ -904,15 +938,15 @@
         z = time.time()
         lineno = 1
         parity = 0
-        
+
         file_revid = self.get_inventory(revid)[file_id].revision
         oldvalues = None
-        
+
         # because we cache revision metadata ourselves, it's actually much
         # faster to call 'annotate_iter' on the weave directly than it is to
         # ask bzrlib to annotate for us.
         w = self._branch.repository.weave_store.get_weave(file_id, self._branch.repository.get_transaction())
-        
+
         revid_set = set()
         for line_revid, text in w.annotate_iter(file_revid):
             revid_set.add(line_revid)
@@ -923,7 +957,7 @@
                                      change=util.Container())
                 return
         change_cache = dict([(c.revid, c) for c in self.get_changes(list(revid_set))])
-        
+
         last_line_revid = None
         for line_revid, text in w.annotate_iter(file_revid):
             if line_revid == last_line_revid:
@@ -941,11 +975,10 @@
             yield util.Container(parity=parity, lineno=lineno, status=status,
                                  change=change, text=util.fixed_width(text))
             lineno += 1
-        
+
         self.log.debug('annotate: %r secs' % (time.time() - z,))
 
     @with_branch_lock
-    @with_bzrlib_read_lock
     def get_bundle(self, revid, compare_revid=None):
         if compare_revid is None:
             parents = self._revision_graph[revid]
@@ -956,4 +989,3 @@
         s = StringIO()
         bzrlib.bundle.serializer.write_bundle(self._branch.repository, revid, compare_revid, s)
         return s.getvalue()
-

=== removed file 'loggerhead/history2.py'
--- loggerhead/history2.py	2007-06-25 10:16:34 +0000
+++ loggerhead/history2.py	1970-01-01 00:00:00 +0000
@@ -1,194 +0,0 @@
-"""Extract data suitable for web presentation from Bazaar branches.
-
-The module is the interface between Loggerhead and bzrlib.  Other code in
-Loggerhead should not use bzrlib APIs, and functions and methods in this
-module should not return bzrlib data types.
-
-The core method is `History.getRevisionInfo`.  This method is meant to be
-cheap, so you should not worry about calling it multiple times with the same
-argument or calling it just to retreive a subset of the information it
-returns.
-
-Revisions are described using good old bzrlib revision ids.
-
-Revision ids, file ids, file names and file paths are always utf-8 encoded 8
-bit strings.
-"""
-
-class FileDelta(object):
-    """Information about how an inventory entry changed in a revision.
-
-    All attribute names are fairly self explanatory.  The ``*_parent`` fields
-    are the file id of the parent or ``None`` for the root.
-
-    :ivar file_id:
-    :ivar old_path:
-    :ivar new_path:
-    :ivar old_name:
-    :ivar new_name:
-    :ivar old_parent:
-    :ivar new_parent:
-    :ivar old_kind:
-    :ivar new_kind:
-    :ivar content_change:
-    :type content_change: ``bool``
-    :ivar execute_change: A boolean reflecting the new state if it changed, or
-                          ``None`` if it did not.
-    :type execute_change: ``bool`` or ``None``
-    """
-
-
-class FileChanges(object):
-    """Contains information about which files changed in a revision.
-
-    Note that the 'files changed' information is relative to left-most parent
-    by default, as in 'bzr log -v'.
-
-    :ivar deltas: A list of `FileDelta` objects.
-    :ivar fileset: ``set(d.file_id for d in self.deltas)``, but cached.
-    """
-
-class RevisionInfo(object):
-    """Contains all the immutable information Loggerhead needs about a
-    particular revision.
-
-    :ivar revid: Obvious.
-    :ivar date: The date and time this revision was committed.
-    :type date: ``datetime.datetime``
-    :ivar committer: The committer.
-    :type committer: utf-8 encoded ``str``
-    :ivar revprops: The branch properties.
-    :type revprops: A dictionary mapping ``str``\ s to ``str``\ s (both utf-8
-                    encoded).
-    :ivar message: The commit message of this revision.
-    :type message: utf-8 encoded ``str``
-    :ivar parents: The list of the revids of the revision's parents.
-    """
-
-class BranchRevisionInfo(object):
-    """Contains all the mutable information Loggerhead needs about a
-    particular revision, and points to the immutable data.
-
-    :ivar revno: The dotted revno of this revision in this branch.
-    :type revno: ``str``
-    :ivar where_merged: The list of revids that have this revision as a
-                        parent.
-    :ivar info: The corresponding `RevisionInfo`.
-    """
-
-class FileEntry(object):
-    """An entry in the list returned by `getFileList`.
-
-    :ivar path: The path of the object in the revision passed to
-                `getFileList`.
-    :ivar fileid: Obvious.
-    :ivar kind: one of ``'file'``, ``'executable'``, ``'link'`` or
-                ``'directory'``.
-    :type kind: ``str``
-    :ivar last_changed_revid: The revid of the revision in which this path
-                              last changed.  Not that this is recursive,
-                              i.e. the last_changed_revid for a directory is
-                              the revid in which the directory or anything
-                              contained in in changed.x
-    :ivar size: The size of this object in this revision in bytes.
-    :type size: ``int``
-    """
-
-class History(object):
-    """Provide the information loggerhead needs about a bzrlib Branch.
-
-    An instance of this class is effectively a wrapper around a
-    `bzrlib.branch.Branch` that translates the information provided by bzrlib
-    into a form convenient for use in loggerhead.
-
-    :ivar last_revision: The revid of the tip of the branch.
-    """
-
-    @classmethod
-    def fromBranch(cls, branch):
-        """Create and initialize a `History` object from a
-        `bzrlib.branch.Branch`.
-        """
-
-    def outOfDate(self):
-        """Decide whether this History object is still current.
-
-        At least to start with, an out of date History object should be thrown
-        away and a fresh one created.
-        """
-
-    def getRevisionInfo(self, revid):
-        """Find the `BranchRevisionInfo` object for a given revision id.
-
-        This operation is to be thought of as 'cheap', which is to say that
-        you should not worry about calling it repeatedly with the same
-        argument or about calling it just to retrieve a subset of the
-        information it includes.
-
-        :param revid: a revision id.
-        :returns: a `BranchRevisionInfo` object or ``None`` if the given
-                  revision id does not exist in the branch.
-        """
-
-    def getChanges(self, revid, other_revid=None):
-        """Find the `FileChanges` object between two revision.
-
-        :param revid: A revision id.
-        :param other_revid: The revision id to compare against.  If ``None``
-                            or not given, compare against the leftmost parent
-                            of ``revid``.
-        :returns: The corresponding `FileChanges` object.
-        """
-
-    # There will need to be some more methods here to allow searching for
-    # revisions by date, committer, commit message, etc...
-
-    def getDiff(self, revidA, revidB):
-        """Compute the diff between two revisions.
-
-        Not sure what this will return yet, probably something similar to but
-        more structured than the 'chunks' loggerhead already works with.
-        """
-
-    def getDirList(self, fileid, revid):
-        """List the files in the given directory as of the given revision.
-
-        :return: a list of `FileEntry` objects.
-        """
-
-    def getFileAnnotation(self, fileid, revid):
-        """Annotate the given file.
-
-        Not sure what this should return, mostly likely a list of
-        (revid-or-None, line contents) (and ``None`` for a binary file).
-        """
-
-    def getFileText(self, fileid, revid):
-        """Fetch the contents of a file at a particular revision.
-
-        :returns: (filename, data)
-        :rtype: ``(str, str)``
-        """
-
-    def getBundle(self, revid, compare_revid=None):
-        """Return a bundle for this revision.
-
-        Relative to compare_revid, or left-most parent if this is not supplied
-        or None.
-
-        :return: A bundle (in a string).
-        """
-
-    def normalizeRevivionSpec(self, revspec):
-        """Convert a revid or revno or 'head' to a revid.
-
-        :returns: a revision id.
-        """
-
-    def normalizeFileArguments(self, revid, fileid, filepath):
-        """Handle various ways of specifying a file in the branch.
-
-        One of ``fileid`` and ``filepath`` should be None.
-
-        :returns: (fileid, filepath)
-        """

=== removed file 'loggerhead/json.py'
--- loggerhead/json.py	2006-12-11 06:44:19 +0000
+++ loggerhead/json.py	1970-01-01 00:00:00 +0000
@@ -1,12 +0,0 @@
-# This module provides helper functions for the JSON part of your
-# view, if you are providing a JSON-based API for your app.
-
-# Here's what most rules would look like:
-# @jsonify.when("isinstance(obj, YourClass)")
-# def jsonify_yourclass(obj):
-#     return [obj.val1, obj.val2]
-# The goal is to break your objects down into simple values:
-# lists, dicts, numbers and strings
-
-from turbojson.jsonify import jsonify
-

=== modified file 'loggerhead/lockfile.py'
--- loggerhead/lockfile.py	2007-01-14 01:20:39 +0000
+++ loggerhead/lockfile.py	2008-02-27 03:36:09 +0000
@@ -35,7 +35,7 @@
     lock may be acquired multiple times by the same thread, as long as it's
     released an equal number of times.  unlike threading.Lock, this lock can
     be used across processes.
-    
+
     this uses os.open(O_CREAT|O_EXCL), which apparently works even on windows,
     but will not work over NFS, if anyone still uses that.  so don't put the
     cache folder on an NFS server...
@@ -50,7 +50,7 @@
             # remove stale locks left over from a previous run
             if time.time() - os.stat(filename).st_mtime > MAX_STALE_TIME:
                 os.remove(filename)
-    
+
     @with_lock
     def _try_acquire(self):
         if self._count > 0:
@@ -63,7 +63,7 @@
             return True
         except OSError:
             return False
-    
+
     def acquire(self):
         # try over and over, sleeping on exponential backoff with an upper limit of about 5 seconds
         pause = 0.1

=== modified file 'loggerhead/lsprof.py'
--- loggerhead/lsprof.py	2006-12-15 01:14:05 +0000
+++ loggerhead/lsprof.py	2008-02-27 03:36:09 +0000
@@ -36,7 +36,7 @@
         for pp in _g_threadmap.values():
             pp.disable()
         threading.setprofile(None)
-    
+
     threads = {}
     for tid, pp in _g_threadmap.items():
         threads[tid] = Stats(pp.getstats(), {})
@@ -112,7 +112,7 @@
         self.out_file = None
 
     def output(self, out_file):
-        self.out_file = out_file        
+        self.out_file = out_file
         print >> out_file, 'events: Ticks'
         self._print_summary()
         for entry in self.data:

=== removed file 'loggerhead/model.py'
--- loggerhead/model.py	2006-12-11 06:44:19 +0000
+++ loggerhead/model.py	1970-01-01 00:00:00 +0000
@@ -1,10 +0,0 @@
-from sqlobject import *
-
-from turbogears.database import PackageHub
-
-hub = PackageHub("loggerhead")
-__connection__ = hub
-
-# class YourDataClass(SQLObject):
-#     pass
-

=== modified file 'loggerhead/static/css/style.css'
--- loggerhead/static/css/style.css	2007-06-25 07:26:04 +0000
+++ loggerhead/static/css/style.css	2007-09-04 01:47:51 +0000
@@ -216,7 +216,7 @@
 }
 
 .log-entries .revision-header {
-    font-size: normal;
+    font-size: 1.0em;
 }
 
 .log-entries .revision-header td {
@@ -315,10 +315,6 @@
     padding-bottom: 2px;
 }
 
-.revision-info table td.date {
-    font-family: monospace;
-}
-
 .revision-info table td.description {
     font-family: monospace;
 }
@@ -404,7 +400,7 @@
 .diff-option-buttons .collapse-file-showall { display: none; }
 
 .diff-block .show-button { display: none; }
-.diff .collapse-style-unified-content { display: none; }
+.collapse-style-unified-content { display: none; }
 .collapse-style-sbs-show { display: none; }
 
 .diff-insert {
@@ -503,6 +499,8 @@
     font-family: monospace;
     font-size: smaller;
     width: 5em;
+    line-height: 1em;
+    vertical-align: bottom;
 }
 
 .inventory td.revision {
@@ -513,7 +511,7 @@
 
 .inventory td.date {
     padding-right: 1em;
-    width: 10em;
+    width: 11em;
 }
 
 .inventory td.changes-link {

=== removed file 'loggerhead/static/css/zptstyle.css'
--- loggerhead/static/css/zptstyle.css	2007-07-04 15:19:45 +0000
+++ loggerhead/static/css/zptstyle.css	1970-01-01 00:00:00 +0000
@@ -1,657 +0,0 @@
-
-/* from launchpad */
-body {
-    background-color: #fcfcfc;
-    color: black;
-    font-family: "Lucida Grande", "Trebuchet MS", sans-serif;
-    font-size: 12pt;
-    padding: 0px;
-    text-align: left;
-}
-
-a {
-    text-decoration: none;
-    color: #000088;
-}
-a:hover {
-    color: black;
-}
-
-img {
-    border: none;
-}
-
-h1 {
-    color: #708a96;
-    font-weight: normal;
-    font-size: 1.6em;
-    margin-bottom: 0em;
-}
-h1 .filename {
-    font-family: monospace;
-}
-
-.branch-name {
-    color: #47626e;
-}
-
-.footer {
-    margin-top: 1em;
-    margin-bottom: 1em;
-}
-.footer .banner {
-    margin-right: 5px;
-}
-.footer .banner img {
-    border: 1px solid #8cacbb;
-}
-
-.collapse-triangle {
-    padding-right: 0.5em;
-}
-
-.hide-all {
-    font-size: smaller;
-    padding: 0.25em 0.75em 0.25em 0.75em;
-}
-
-
-
-/* ----- navigation bar at the top ----- */
-
-.navbar {
-}
-
-.bar table {
-    width: 100%;
-    border-collapse: collapse;
-}
-
-/* safari gives forms a big default margin, to vex me */
-.bar form {
-    margin: 0px;
-}
-
-.bar {
-    background-color: #dee7ec;
-    border: 1px solid #8cacbb;
-    font-size: smaller;
-    padding-top: 0px;
-    padding-bottom: 0px;
-}
-
-/* revision navigation buttons (the ones at the top right) */
-.navbuttons {
-    padding: 2px;
-    bottom: 0px;
-    text-align: right;
-    width: auto;
-    border-left: 1px dotted #8cacbb;
-    padding-left: 0.5em;
-    padding-right: 0.5em;
-}
-
-.navbuttons a {
-    font-weight: normal;
-    margin-left: 2px;
-    margin-right: 2px;
-}
-
-.navbuttons a:hover {
-    padding: 0;
-    color: blue;
-}
-
-/* navigation button in the pages (the ones at the top left, and bottom) */
-.buttons {
-}
-
-.buttons a,.buttons .search {
-    font-weight: normal;
-    color: #000088;
-    font-size: 12pt;
-    padding-left: 10px;
-    padding-right: 7px;
-    border-right: 1px dotted #8cacbb;
-}
-
-.buttons a:hover,.rbuttons a:hover {
-    font-weight: normal;
-    color: blue;
-}
-
-.feed img {
-    vertical-align: text-top;
-}
-
-.rbuttons a {
-    font-weight: normal;
-    color: #000088;
-    font-size: 12pt;
-    padding-right: 7px;
-    padding-left: 10px;
-    border-left: 1px dotted #8cacbb;
-    text-align: right;
-    width: auto;
-}
-
-
-/* ----- revision display in various places ----- */
-
-.revision .revno {
-    font-weight: bold;
-}
-
-.revision .revid {
-    font-family: monospace;
-    font-size: smaller;
-    padding-left: 1em;
-}
-
-.revision .log {
-    font-size: smaller;
-}
-
-
-/* ---- sub-links underneath the headline on some pages ----- */
-
-div.links {
-    color: #708a96;
-    padding-top: 0.5em;
-    padding-left: 3em;
-    font-size: 0.8em;
-    margin-bottom: 2em;
-}
-
-div.links div {
-    margin-bottom: 3px;
-}
-
-
-/* ----- list of merge points for a revision ----- */
-
-.info-entry {
-    font-size: smaller;
-    margin-bottom: 2em;
-    padding: 0px;
-    background-color: #f4f5f8;
-    border: 1px solid #8cacbb;
-    width: 60%;
-}
-
-.info-entry th {
-    background-color: #dee7ec;
-    font-weight: normal;
-    color: black;
-    text-align: right;
-    vertical-align: top;
-
-    padding-left: 0.5em;
-    padding-right: 0.5em;
-    padding-top: 2px;
-    padding-bottom: 2px;
-}
-
-.info-entry td {
-    font-weight: normal;
-    padding-left: 0.5em;
-    padding-top: 2px;
-    padding-bottom: 2px;
-}
-
-
-/* ----- changelog page ----- */
-
-/* secret css-jutsu to make hidden margins vanish on a table */
-.log-entries {
-    border-collapse: collapse;
-    width: 98%;
-    margin-top: 0.5em;
-    margin-left: 1em;
-}
-
-.log-entries .log-header {
-    background-color: #8cacbb;
-    padding-top: 3px;
-    padding-bottom: 3px;
-    text-align: left;
-}
-
-.log-entries .revision-header {
-    font-size: normal;
-}
-
-.log-entries .revision-header td {
-    padding-top: 2px;
-    padding-bottom: 2px;
-}
-
-.log-entries .revision-header td.expand-button {
-    width: 20px;
-}
-
-.log-entries .revision-number {
-    padding-left: 0.5em;
-}
-
-.log-entries .revision-header td.date {
-    font-size: smaller;
-}
-
-.log-entries .revision-header td.author {
-    font-size: smaller;
-    padding-right: 0.5em;
-}
-
-.log-entries .revision-header .inventory-link {
-    font-size: smaller;
-    vertical-align: middle;
-    text-align: left;
-    padding-right: 0.5em;
-}
-
-.log-entries table.revision-details {
-    font-size: smaller;
-    margin-bottom: 1.0em;
-}
-
-.log-entries table.revision-details th {
-    text-align: right;
-    padding-right: 0.5em;
-    vertical-align: top;
-}
-
-.log-entries table.revision-details td.files .filename {
-    font-family: monospace;
-}
-
-.log-entries .revision-details td.files .jump {
-    font-size: smaller;
-}
-
-.log-entries .revision-details td.description {
-    font-family: monospace;
-}
-
-/* default state: hide all */
-.revision-header .hide-button { display: none; }
-.hidden-details { display: none; }
-.changelog .collapse-cl-hideall { display: none; }
-
-
-/* ----- revision page ----- */
-
-.revision-info {
-    border: none;
-    font-size: smaller;
-    margin-bottom: 1em;
-    padding: 0;
-    background-color: #f4f5f8;
-    border: 1px solid #8cacbb;
-    width: 80%;
-}
-
-/* secret css-jutsu to make hidden margins vanish on a table */
-.revision-info table {
-    border-collapse: collapse;
-    width: 100%;
-}
-
-.revision-info th {
-    background-color: #dee7ec;
-    color: black;
-    font-weight: normal;
-    text-align: right;
-    vertical-align: top;
-    
-    padding-left: 0.5em;
-    padding-right: 0.5em;
-    padding-top: 2px;
-    padding-bottom: 2px;
-}
-
-.revision-info td {
-    font-weight: normal;
-    padding-left: 0.5em;
-    padding-top: 2px;
-    padding-bottom: 2px;
-}
-
-.revision-info table td.date {
-    font-family: monospace;
-}
-
-.revision-info table td.description {
-    font-family: monospace;
-}
-
-.revision-info table td.files .filename {
-    font-family: monospace;
-}
-
-.revision-info table td.files a.jump {
-    font-size: smaller;
-}
-    
-.revision-info table tr.divider {
-    border-top: 1px solid #8cacbb;
-}
-
-
-/* ----- overly complex diff display ----- */
-
-.diff {
-    margin-bottom: 1em;
-    padding-top: 10px;
-    width: 100%;
-    background-color: #fcfcfc;
-    font-family: monospace;
-}
-
-
-.diff-block {
-    border-collapse: collapse;
-    width: 100%;
-    margin-bottom: 1em;
-}
-
-.diff-block th.filename {
-    font-family: monospace;
-    font-weight: normal;
-    font-size: smaller;
-    padding-left: 1em;
-    padding-top: 1em;
-    padding-bottom: 1em;
-    text-align: left;
-    background-color: #dee7ec;
-}
-
-.diff-block .diff-chunk {
-    font-size: 66%;
-    margin-top: 0px;
-    margin-bottom: 2em;
-}
-
-.diff-block th.lineno {
-    font-weight: bold;
-    border-bottom: 1px solid #8cacbb;
-    border-top: 1px solid #dee7ec;
-}
-
-.diff-block .diff-spacing {
-    line-height: 1em;
-}
-
-.diff-option-buttons {
-    margin-top: 1.0em;
-    margin-bottom: 1.0em;
-    border-collapse: collapse;
-}
-.diff-option-buttons td {
-    line-height: 1em;
-    height: 1em;
-    vertical-align: middle;
-    padding: 0px;
-}
-.diff-option-buttons .label {
-    font-size: smaller;
-    padding-left: 0.75em;
-    padding-right: 2em;
-    line-height: 1em;
-}
-.diff-option-buttons .spacey {
-    padding-right: 5em;
-}
-
-.diff-option-buttons .collapse-file-showall { display: none; }
-
-.diff-block .show-button { display: none; }
-.diff .collapse-style-unified-content { display: none; }
-.collapse-style-sbs-show { display: none; }
-
-.diff-insert {
-    background-color: #9cc8ff;
-}
-.diff-delete {
-    background-color: #ffa0a0;
-}
-.diff-chunk td.text {
-    padding-left: 1em;
-}
-
-.diff-key {
-    display: inline;
-    font-size: smaller;
-}
-
-.diff-key-block {
-    width: 1em;
-    height: 1em;
-    border: 1px solid #8cacbb;
-}
-
-.diff-key .label {
-    padding-right: 2em;
-}
-
-.diff .lineno {
-    color: purple;
-    background-color: #dee7ec; 
-    width: 4em;
-    text-align: right;
-    padding-right: 3px;
-    border-bottom: 1px solid #8cacbb;
-}
-.diff .lineno-skip {
-    width: 4em;
-    background-color: #dee7ec;
-}
-.diff .info {
-    color: blue;
-    background-color: #dee7ec;
-    margin-top: 15px;
-}
-/*
-.diff-context-skip {
-    background-color: #dee7ec;
-}*/
-
-
-/* ----- file inventory ----- */
-
-.inventory-path {
-    margin-top: 2em;
-}
-
-table.inventory {
-    border-collapse: collapse;
-    width: 100%;
-    font-size: smaller;
-    margin-top: 1em;
-    border: 1px solid #8cacbb;
-}
-
-table.inventory td {
-    padding-top: 3px;
-    padding-bottom: 3px;
-    vertical-align: middle;
-}
-
-table.inventory th {
-    background-color: #8cacbb;
-    padding-top: 3px;
-    padding-bottom: 3px;
-    text-align: left;
-}
-
-table.inventory th.permissions {
-    padding-left: 0.5em;
-}
-
-.parity0 { background-color: #dee7ec; }
-.parity1 { background-color: #eff8fd; }
-
-.inventory td.permissions {
-    font-family: monospace;
-    padding-left: 0.5em;
-    width: 9em;
-}
-
-.inventory td.filename {
-    font-family: monospace;
-}
-
-.inventory td.size {
-    font-family: monospace;
-    font-size: smaller;
-    width: 5em;
-}
-
-.inventory td.revision {
-    padding-right: 1em;
-    text-align: left;
-    width: 8em;
-}
-
-.inventory td.date {
-    padding-right: 1em;
-    width: 10em;
-}
-
-.inventory td.changes-link {
-    font-size: smaller;
-    width: 6em;
-}
-
-.inventory td.download-link {
-    font-size: smaller;
-    width: 8em;
-}
-
-span.folder {
-    font-family: monospace;
-}
-
-
-/* ----- file annotation ----- */
-
-.annotate-path {
-    font-family: monospace;
-}
-
-.annotate table {
-    border-collapse: collapse;
-    width: 100%;
-    font-size: 66%;
-}
-
-.annotate td {
-    padding-top: 2px;
-    padding-bottom: 2px;
-    vertical-align: bottom;
-}
-
-.annotate td.lineno {
-    font-family: monospace;
-    text-align: right;
-    width: 4em;
-    padding-right: 5px;
-    font-weight: bold;
-    border-bottom: 1px solid #8cacbb;
-}
-
-.annotate td.revno {
-    font-family: monospace;
-    text-align: right;
-    width: 8em;
-    padding-right: 5px;
-    border-right: 1px solid #8cacbb;
-}
-
-.annotate td.text {
-    font-family: monospace;
-    padding-left: 1em;
-}
-
-.annotate th {
-    background-color: #8cacbb;
-    padding-top: 3px;
-    padding-bottom: 3px;
-    text-align: left;
-}
-
-.annotate th.lineno {
-    text-align: right;
-}
-
-.annotate th.revision {
-    text-align: right;
-    padding-right: 5px;
-}
-
-.annotate th.text {
-    padding-left: 1em;
-}
-
-
-/* ----- top-level browse page ----- */
-
-.browse-project-name {
-    background-color: #dee7ec;
-    padding-top: 5px;
-    padding-bottom: 5px;
-    margin-left: 1em;
-    margin-right: 1em;
-    padding-left: 1em;
-    width: 90%;
-    font-weight: bold;
-}
-
-.browse-project-description {
-	padding-top: 1em;
-	padding-bottom: 1em;
-	margin-left: 2em;
-    width: 90%;
-}
-	
-.browse-view {
-	margin-top: 1em;
-	margin-bottom: 1em;
-	margin-left: 2em;
-}
-
-.browse-view table {
-    border-collapse: collapse;
-    width: 90%;
-    margin-bottom: 2em;
-}
-
-.browse-view table tr.heading {
-    background-color: #8cacbb;
-    border-bottom: solid 1px #8cacbb;
-    font-size: smaller;
-}
-
-.browse-view td,.browse-view th {
-    padding: 0.25em 0.5em 0.25em 0.5em;
-}
-
-.browse-view td.name {
-    width: 20%;
-}
-
-.browse-view td.last-update {
-    font-size: smaller;
-    width: 20em;
-}
-
-.browse-view td.url {
-    font-family: monospace;
-    font-size: smaller;
-    padding-left: 3em;
-    padding-top: 0px;
-}
-
-.browse-view td.feed {
-	width: 16px;
-}

=== modified file 'loggerhead/templates/annotate.kid'
--- loggerhead/templates/annotate.kid	2007-06-21 16:39:33 +0000
+++ loggerhead/templates/annotate.kid	2007-09-04 17:06:03 +0000
@@ -31,7 +31,7 @@
             <td class="lineno ${line.status}"><a name="L${line.lineno}"> ${line.lineno} </a></td>
             <td class="revno ${line.status}">
                 <a py:if="line.status=='changed'" href="${branch.url('/revision', **util.get_context(clear=1, start_revid=line.change.revid, file_id=file_id))}"
-                    title="${line.change.revno} by ${util.hide_email(line.change.author)}, on ${line.change.date.strftime('%d %b %Y %H:%M')} (${util.ago(line.change.date)})">${util.trunc(line.change.revno)}</a>
+                    title="${line.change.revno} by ${util.hide_email(line.change.author)}, on ${util.format_date(line.change.date)} (${util.ago(line.change.date)})">${util.trunc(line.change.revno)}</a>
             </td>
             <td class="text ${line.status}"> ${line.text} </td>
         </tr>

=== removed file 'loggerhead/templates/annotate.pt'
--- loggerhead/templates/annotate.pt	2007-07-05 09:10:39 +0000
+++ loggerhead/templates/annotate.pt	1970-01-01 00:00:00 +0000
@@ -1,100 +0,0 @@
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
-          "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-<html xmlns="http://www.w3.org/1999/xhtml">
-  <head>
-    <meta content="text/html; charset=utf-8" http-equiv="Content-Type"/>
-    <title>
-      <tal:branch-name content="branch/friendly_name">
-        ${branch.friendly_name}
-      </tal:branch-name>
-      : contents of
-      <tal:path content="path">
-        ${path}
-      </tal:path>
-      at revision
-      <tal:revno content="change/revno">
-        ${change.revno}
-      </tal:revno>
-    </title>
-    <link rel="stylesheet"
-          tal:attributes="href python:tg.url('/static/css/zptstyle.css')" />
-  </head>
-
-  <body>
-
-    <tal:navigation content="structure here/navigation.pt" />
-
-    <h1>
-      <span class="branch-name" tal:content="branch/friendly_name">
-        ${branch.friendly_name}
-      </span>
-      :
-      <span class="annotate-path" tal:content="path">
-        ${path}
-      </span>
-      (revision <tal:revno content="change/revno">${change.revno}</tal:revno>)
-    </h1>
-    <div class="links">
-      <div>
-        <b>&#8594;</b>
-        <a tal:attributes="href python:branch.url([ '/files', revid ], **util.get_context(clear=1))">
-          browse files
-        </a>
-      </div>
-      <div>
-        <b>&#8594;</b>
-        <a tal:attributes="href python:branch.url('/revision', **util.get_context(clear=1, start_revid=revid))">
-          view revision
-        </a>
-      </div>
-      <div>
-        <b>&#8594;</b>
-        <a tal:attributes="href python:branch.url('/changes', **util.get_context(clear=1, start_revid=revid, file_id=file_id))">
-          view changes to this file
-        </a>
-      </div>
-      <div>
-        <b>&#8594;</b>
-        <a tal:attributes="href python:branch.url([ '/download', revid, file_id, filename ])">
-          download file
-        </a>
-      </div>
-    </div>
-
-    <div class="annotate">
-      <table>
-        <tr>
-          <th class="lineno"> Line# </th>
-          <th class="revision"> Revision </th>
-          <th class="text"> Contents </th>
-        </tr>
-
-        <tr tal:repeat="line contents"
-            tal:attributes="class string:parity${line/parity}">
-          <td tal:attributes="class string:lineno ${line/status}">
-            <a tal:attributes="name string:L${line/lineno}"
-               tal:content="line/lineno">
-              ${line.lineno}
-            </a>
-          </td>
-          <td tal:attributes="class string:revno ${line/status}">
-            <a tal:condition="python:line.status=='changed'"
-               tal:content="python:util.trunc(line.change.revno)"
-               tal:attributes="href python:branch.url('/revision', **util.get_context(clear=1, start_revid=line.change.revid, file_id=file_id));
-                               title python:'%s by %s, on %s (%s)'%(line.change.revno, util.hide_email(line.change.author), line.change.date.strftime('%d %b %Y %H:%M'), util.ago(line.change.date))">
-
-              ${util.trunc(line.change.revno)}
-            </a>
-          </td>
-          <td tal:attributes="class string:text ${line/status}"
-              tal:content="line/text">
-            ${line.text}
-          </td>
-        </tr>
-      </table>
-    </div>
-
-    <tal:footer content="structure here/footer.pt" />
-
-  </body>
-</html>

=== modified file 'loggerhead/templates/browse.kid'
--- loggerhead/templates/browse.kid	2007-06-08 11:52:33 +0000
+++ loggerhead/templates/browse.kid	2008-02-27 01:55:39 +0000
@@ -34,7 +34,7 @@
             <tr>
             <td class="name"> <a href="${tg.url([ '/' + project.name, view.name ])}">${view.friendly_name}</a> </td>
             <td class="description"> ${view.description} </td>
-            <td class="last-update"> ${view.get_history().last_revid and util.ago(view.last_updated()) or "(no revisions)"} </td>
+            <td class="last-update"> ${view.get_history().has_revisions and util.ago(view.last_updated()) or "(no revisions)"} </td>
             <td class="feed">
                 <a href="${tg.url([ '/' + project.name, view.name, 'atom' ])}" title="atom feed">
                     <img src="${tg.url('/static/images/feed-icon-16x16.gif')}" alt="atom feed" />

=== modified file 'loggerhead/templates/changelog.kid'
--- loggerhead/templates/changelog.kid	2007-06-26 05:34:28 +0000
+++ loggerhead/templates/changelog.kid	2007-09-04 17:21:59 +0000
@@ -14,7 +14,22 @@
         <a href="${branch.url([ '/annotate', revid ], **util.get_context(file_id=file_id))}" title="Annotate ${filename}">${filename}</a>
     </span>
     
+    <span py:def="file_diff_link(revid, filename)">
+        <a href="javascript:diff_url('${branch.context_url([ '/revision', revid ]) + '#' + filename}')"
+           title="View diff for ${filename}">${filename}</a>
+    </span>
+    
     ${use_collapse_buttons()}
+
+    <script type="text/javascript"> <!--
+    function diff_url(url) {
+        if (document.cookie.indexOf('diff=unified') >= 0) {
+            this.location.href = url + "-u";
+        } else {
+            this.location.href = url + "-s";
+        }
+    }
+    // --> </script>
 </head>
 
 <body onload="javascript:sortCollapseElements();">
@@ -62,7 +77,7 @@
             <span py:strip="True" py:if="not all_same_author">
                 <td class="author"> ${util.trunc(util.hide_email(entry.author), 20)} </td>
             </span>
-            <td class="date"> ${entry.date.strftime('%Y-%m-%d, %H:%M')} &nbsp; (${util.ago(entry.date)}) </td>
+            <td class="date"> ${util.format_date(entry.date)} &nbsp; (${util.ago(entry.date)}) </td>
             <td class="inventory-link"> 
                 <a href="${branch.url([ '/files', entry.revid ])}"
                     title="Files at revision ${entry.revno}"> files</a>
@@ -83,9 +98,9 @@
                 <tr py:if="len(entry.parents) > 1">
                     <th class="parents"> merged from: </th>
                     <td class="parents">
-                        <span py:for="parent in entry.parents"><span py:if="parent.revid != entry.parents[0].revid">
-                            ${loglink(parent.revid, '(' + parent.revno + util.if_present(' %s', parent.branch_nick) + ')')} <br />
-                        </span></span>
+                        <span py:for="parent in entry.parents[1:]">
+                            ${loglink(parent.revid, parent.revno + util.if_present(' (%s)', parent.branch_nick))} <br />
+                        </span>
                     </td>
                 </tr>
                 <span py:strip="True" py:if="all_same_author">
@@ -115,8 +130,7 @@
                 <tr py:if="entry.changes.modified">
                     <th class="files"> files modified: </th>
                     <td class="files"> <span py:for="item in entry.changes.modified">
-                        <span class="filename">${file_link(item.filename, item.file_id, entry.revid)}</span> &nbsp;
-                        <a href="${branch.url([ '/revision', entry.revid ], **util.get_context()) + '#' + item.filename}" class="jump">&#8594; diff</a>
+                        <span class="filename">${file_diff_link(entry.revid, item.filename)}</span>
                         <br />
                     </span> </td>
                 </tr>

=== removed file 'loggerhead/templates/footer.pt'
--- loggerhead/templates/footer.pt	2007-07-04 15:55:56 +0000
+++ loggerhead/templates/footer.pt	1970-01-01 00:00:00 +0000
@@ -1,12 +0,0 @@
-<div class="footer">
-  <span class="banner">
-    <a href="http://www.bazaar-vcs.org/">
-      <img tal:attributes="src python:tg.url('/static/images/bazaar-banner.png')" />
-    </a>
-  </span>
-  <span class="banner">
-    <a href="http://www.lag.net/loggerhead/">
-      <img tal:attributes="src python:tg.url('/static/images/loggerhead-banner.png')" />
-    </a>
-  </span>
-</div>
\ No newline at end of file

=== modified file 'loggerhead/templates/inventory.kid'
--- loggerhead/templates/inventory.kid	2007-06-25 06:44:52 +0000
+++ loggerhead/templates/inventory.kid	2007-09-04 17:06:03 +0000
@@ -12,9 +12,9 @@
 
 <h1> <span class="branch-name">${branch.friendly_name}</span> : files for revision ${change.revno}
 	<div class="links">
-	    <div> <b>&#8594;</b> <a href="${branch.url('/revision', **util.get_context(clear=1, start_revid=revid))}">
+	    <div> <b>&#8594;</b> <a href="${branch.context_url('/revision', clear=1, start_revid=revid)}">
 	        view revision </a> </div>
-	    <div> <b>&#8594;</b> <a href="${branch.url('/changes', **util.get_context(clear=1, start_revid=revid))}">
+	    <div> <b>&#8594;</b> <a href="${branch.context_url('/changes', clear=1, start_revid=revid)}">
 	        view branch changes </a> </div>
 	</div>
 </h1>
@@ -28,7 +28,7 @@
         </tr>
         <tr>
             <th class="date">date:</th>
-            <td class="date"> ${change.date.strftime('%d %b %Y %H:%M')} </td>
+            <td class="date"> ${util.format_date(change.date)} </td>
         </tr>
 
         <tr py:if="len(change.merge_points) > 0">
@@ -62,35 +62,34 @@
 <table class="inventory" width="100%">
     <tr class="header">
         <th class="permissions"> Permissions </th>
-        <th> <a href="${branch.url([ '/files', revid ], **util.get_context(sort='filename'))}">Filename</a> </th>
+        <th> <a href="${branch.context_url([ '/files', revid ], sort='filename')}">Filename</a> </th>
         <th> Latest Rev </th>
-        <th> <a href="${branch.url([ '/files', revid ], **util.get_context(sort='date'))}">Last Changed</a> </th>
-        <th> <a href="${branch.url([ '/files', revid ], **util.get_context(sort='size'))}">Size</a> </th>
+        <th> <a href="${branch.context_url([ '/files', revid ], sort='date')}">Last Changed</a> </th>
+        <th> <a href="${branch.context_url([ '/files', revid ], sort='size')}">Size</a> </th>
         <th>  </th>
         <th>  </th>
     </tr>
     
     <tr class="parity1" py:if="updir">
         <td class="permissions">drwxr-xr-x</td>
-        <td class="filename directory"><a href="${branch.url([ '/files', revid ],
-            **util.get_context(file_id=updir_file_id))}"> (up) </a></td>
+        <td class="filename directory"><a href="${branch.context_url([ '/files', revid ], file_id=updir_file_id)}"> (up) </a></td>
         <td> </td> <td> </td> <td> </td> <td> </td>
     </tr>
 
     <tr py:for="file in filelist" class="parity${file.parity}">
         <td class="permissions"> ${util.fake_permissions(file.kind, file.executable)} </td>
         <td class="filename ${file.kind}">
-            <a py:if="file.kind=='directory'" href="${branch.url([ '/files', revid ],
-                **util.get_context(file_id=file.file_id))}">${file.filename}/</a>
+            <a py:if="file.kind=='directory'" href="${branch.context_url([ '/files', revid ],
+                file_id=file.file_id)}">${file.filename}/</a>
             <span py:if="file.kind=='symlink'">${file.filename}@</span>
-            <a py:if="file.kind=='file'" href="${branch.url([ '/annotate', revid ],
-                **util.get_context(file_id=file.file_id))}" title="Annotate ${file.filename}">${file.filename}</a>
+            <a py:if="file.kind=='file'" href="${branch.context_url([ '/annotate', revid ],
+                file_id=file.file_id)}" title="Annotate ${file.filename}">${file.filename}</a>
         </td>
         <td class="revision"> ${revision_link(file.revid, util.trunc(file.change.revno, 15), **util.get_context(start_revid=file.revid, file_id=file.file_id))} </td>
-        <td class="date"> ${file.change.date.strftime('%Y-%m-%d, %H:%M')} </td>
+        <td class="date"> ${util.format_date(file.change.date)} </td>
         <td class="size"> <span py:if="file.kind=='file'"> ${util.human_size(file.size)} </span></td>
         <td class="changes-link"> 
-            <a href="${branch.url('/changes', **util.get_context(start_revid=file.revid, file_id=file.file_id))}"
+            <a href="${branch.context_url('/changes', start_revid=file.revid, file_id=file.file_id)}"
                title="Changes affecting ${file.filename} up to revision ${file.change.revno}"> changes </a>
         </td>
         <td class="download-link">

=== modified file 'loggerhead/templates/master.kid'
--- loggerhead/templates/master.kid	2007-01-24 19:17:40 +0000
+++ loggerhead/templates/master.kid	2007-09-02 22:54:43 +0000
@@ -19,8 +19,8 @@
               file_id=getattr(navigation, 'file_id', None))}"><table><tr>
             <td><span class="buttons">
                 <!-- ! navbar buttons never change, from now on.  i decree it! -->
-                <a href="${branch.url('/changes', **util.get_context(clear=1))}"> changes </a>
-                <a href="${branch.url('/files', **util.get_context(clear=1))}"> files </a>
+                <a href="${branch.context_url('/changes', clear=1)}"> changes </a>
+                <a href="${branch.context_url('/files', clear=1)}"> files </a>
                 <span class="search"> search: <input type="text" name="q" /> </span>
             </span></td>
             <td align="right" py:if="hasattr(navigation, 'revid_list')">
@@ -34,8 +34,7 @@
                     </span>
                 	<span py:if="not navigation.prev_page_revid"> &#171; </span>
                     revision ${history.get_revno(revid)}
-                    (<span py:if="navigation.pagesize > 1">page </span>
-                    ${navigation.page_position} / ${navigation.page_count})
+                    (<span py:if="navigation.pagesize > 1">page </span>${navigation.page_position} / ${navigation.page_count})
                 	<span py:if="navigation.next_page_revid">
                     <a href="${navigation.next_page_url}" title="Next page"> &#187; </a>
                     </span>

=== removed file 'loggerhead/templates/master.pt'
--- loggerhead/templates/master.pt	2007-07-04 14:51:40 +0000
+++ loggerhead/templates/master.pt	1970-01-01 00:00:00 +0000
@@ -1,120 +0,0 @@
-<metal:page define-macro="master">
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
-          "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-<html xmlns="http://www.w3.org/1999/xhtml">
-
-  <head>
-    <meta content="text/html; charset=UTF-8" http-equiv="content-type" tal:replace="nothing"/>
-    <title tal:content="view/title">Your title goes here</title>
-    <meta py:replace="item[:]"/>
-    <style type="text/css" media="screen"
-           tal:content="string:@import url(${tg.url('/static/css/style.css')});">
-@import url(/static/css/style.css);
-    </style>
-
-<!-- !define common navbar -->
-<span py:def="navbar()" py:strip="True">
-    <!-- !requires: ${navigation: start_revid, revid, revid_list, pagesize, buttons, scan_url}, ${branch}, ${history} -->
-    <div class="bar" py:if="navigation is not None">
-        <!-- form must go OUTSIDE the table, or safari will add extra padding :( -->
-        <form action="${branch.url('/changes', start_revid=getattr(navigation, 'start_revid', None),
-              file_id=getattr(navigation, 'file_id', None))}"><table><tr>
-            <td><span class="buttons">
-                <!-- ! navbar buttons never change, from now on.  i decree it! -->
-                <a href="${branch.url('/changes', **util.get_context(clear=1))}"> changes </a>
-                <a href="${branch.url('/files', **util.get_context(clear=1))}"> files </a>
-                <span class="search"> search: <input type="text" name="q" /> </span>
-            </span></td>
-            <td align="right" py:if="hasattr(navigation, 'revid_list')">
-                <span py:if="hasattr(navigation, 'feed')" class="rbuttons feed">
-                    <a href="${branch.url('/atom')}">
-                    <img src="${tg.url('/static/images/feed-icon-16x16.gif')}" /></a>
-                </span>
-                <span class="navbuttons">
-                	<span py:if="navigation.prev_page_revid">
-                	<a href="${navigation.prev_page_url}" title="Previous page"> &#171; </a>
-                    </span>
-                	<span py:if="not navigation.prev_page_revid"> &#171; </span>
-                    revision ${history.get_revno(revid)}
-                    (<span py:if="navigation.pagesize > 1">page </span>
-                    ${navigation.page_position} / ${navigation.page_count})
-                	<span py:if="navigation.next_page_revid">
-                    <a href="${navigation.next_page_url}" title="Next page"> &#187; </a>
-                    </span>
-                	<span py:if="not navigation.next_page_revid"> &#187; </span>
-                </span>
-            </td>
-        </tr></table></form>
-    </div>
-</span>
-
-<metal:revlink metal:define-macro="revision-link">
-  <tal:comment replace="nothing">
-    This macro expects the following variables:
-    :revid:
-    :text:
-    :history:
-    :branch:
-    :overrides: ???
-  </tal:comment>
-    <a tal:attributes="title string:Show revision ${history.get_revno(revid)}
-                       href string:${branch.url([ '/revision', revid ], **util.get_context(**overrides))}"
-       tal:content="text"
-       href="/branch/revision/$revid">
-      $text
-    </a>
-  </metal:revlink>
-
-<!-- ! expand button functions: -->
-
-	<span py:strip="True" py:def="use_collapse_buttons()">
-		<!-- this is totally matty's fault.  i don't like javacsript. ;) -->
-		<script type="text/javascript" src="${tg.url('/static/javascript/collapse.js')}"></script>
-	</span>
-	
-	<span py:strip="True" py:def="collapse_all_button(group, normal='block')">
-	    <a class="hide-all collapse-${group}-hideall"
-	       href="javascript:collapseAllDisplay('${group}','none')">
-	        <img src="${tg.url('/static/images/nav-small-down.gif')}"
-	             width="10" height="10" alt="collapse"
-	             class="collapse-triangle" />collapse all</a>
-	    <a class="hide-all collapse-${group}-showall"
-	       href="javascript:collapseAllDisplay('${group}','${normal}')">
-	        <img src="${tg.url('/static/images/nav-small-right.gif')}"
-	             width="10" height="10" alt="expand"
-	             class="collapse-triangle" />expand all</a>
-	</span>
-	
-	<span py:strip="True" py:def="collapse_button(group, name, normal='block')">
-	    <a href="javascript:collapseDisplay('${group}','${name}','none')"
-	       class="hide-button collapse-${group}-${name}-hide" title="collapse">
-	        <img src="${tg.url('/static/images/nav-small-down.gif')}"
-	             width="10" height="10" alt="collapse"
-	             class="collapse-triangle" />
-	    </a>
-	    <a href="javascript:collapseDisplay('${group}','${name}','${normal}')"
-	       class="show-button collapse-${group}-${name}-show" title="expand">
-	    	<img src="${tg.url('/static/images/nav-small-right.gif')}"
-	    	     witdh="10" height="10" alt="expand"
-	    	     class="collapse-triangle" />
-	    </a>
-	</span>
-
-</head>
-
-<body py:match="item.tag=='{http://www.w3.org/1999/xhtml}body'"
-      py:attrs="item.items()">
-
-    <div py:replace="[item.text]+item[:]"/>
-
-	<!-- End of main_content -->
-	
-	<div class="footer">
-	    <span class="banner"><a href="http://www.bazaar-vcs.org/">
-	    <img src="${tg.url('/static/images/bazaar-banner.png')}" /></a></span>
-	    <span class="banner"><a href="http://www.lag.net/loggerhead/">
-	    <img src="${tg.url('/static/images/loggerhead-banner.png')}" /></a></span>
-	</div>
-</body>
-
-</html>

=== removed file 'loggerhead/templates/navigation.pt'
--- loggerhead/templates/navigation.pt	2007-07-04 16:00:18 +0000
+++ loggerhead/templates/navigation.pt	1970-01-01 00:00:00 +0000
@@ -1,37 +0,0 @@
-<div class="bar">
-  <form tal:attributes="action python:branch.url('/changes', start_revid=getattr(navigation, 'start_revid', None), file_id=getattr(navigation, 'file_id', None))">
-    <table>
-      <tr>
-        <td>
-          <span class="buttons">
-            <a href="${branch.url('/changes', **util.get_context(clear=1))}"> changes </a>
-            <a href="${branch.url('/files', **util.get_context(clear=1))}"> files </a>
-            <span class="search">
-              search: <input type="text" name="q" />
-            </span>
-          </span>
-        </td>
-        <td align="right" tal:condition="navigation/revid_list|nothing">
-          <span tal:condition="navigation/feed" class="rbuttons feed">
-            <a href="${branch.url('/atom')}">
-              <img src="${tg.url('/static/images/feed-icon-16x16.gif')}" />
-            </a>
-          </span>
-          <span class="navbuttons">
-            <span tal:condition="navigation/prev_page_revid">
-              <a href="${navigation.prev_page_url}" title="Previous page"> &#171; </a>
-            </span>
-            <span tal:condition="python:not navigation.prev_page_revid"> &#171; </span>
-            revision ${history.get_revno(revid)}
-            (<span tal:condition="python:navigation.pagesize > 1">page </span>
-            ${navigation.page_position} / ${navigation.page_count})
-            <span tal:condition="navigation/next_page_revid">
-              <a href="${navigation.next_page_url}" title="Next page"> &#187; </a>
-            </span>
-            <span tal:condition="python:not navigation.next_page_revid"> &#187; </span>
-          </span>
-        </td>
-      </tr>
-    </table>
-  </form>
-</div>

=== modified file 'loggerhead/templates/revision.kid'
--- loggerhead/templates/revision.kid	2007-06-26 06:28:36 +0000
+++ loggerhead/templates/revision.kid	2007-09-04 17:06:03 +0000
@@ -17,24 +17,33 @@
     <span py:replace="use_collapse_buttons()"></span>
     
     <script type="text/javascript"> <!--
-    function show_sbs() {
+    function show_sbs(path) {
         collapseDisplay('style', 'sbs', 'table');
         collapseDisplay('style', 'unified', 'none');
-        document.cookie='diff=sbs';
+        document.cookie='diff=sbs; path=' + path; 
     }
-    function show_unified() {
+    function show_unified(path) {
         collapseDisplay('style', 'unified', 'table');
         collapseDisplay('style', 'sbs', 'none');
-        document.cookie='diff=unified'; 
+        document.cookie='diff=unified; path=' + path; 
     }
-    function load() {
+    function load(path) {
         sortCollapseElements();
-        if (document.cookie.indexOf('diff=unified') >= 0) { show_unified(); }
+        if (document.cookie.indexOf('diff=unified') >= 0) {
+            show_unified(path);
+        }
+        if (document.cookie.indexOf('stop=1') >= 0) {
+            return;
+        }
+        if (document.cookie.indexOf('diff=unified') >= 0) {
+            window.location.hash = window.location.hash;
+            document.cookie='stop=1; max-age=5; path=' + window.location.pathname;
+        } 
     }
     // --> </script>
 </head>
 
-<body onload="javascript:load()">
+<body onload="javascript:load('${branch.url('/')}')">
 
 ${navbar()}
 
@@ -42,32 +51,27 @@
     <span py:if="compare_revid is not None"> (compared to revision ${history.get_revno(compare_revid)}) </span>
     
     <div class="links">
-        <div> <b>&#8594;</b> <a href="${branch.url([ '/files', revid ], **util.get_context())}">
-            browse files</a> </div>
-        <div> <b>&#8594;</b> <a href="${branch.url('/changes', **util.get_context(start_revid=revid))}">
-            view branch changes</a> </div>
-        <span py:if="compare_revid is None" py:strip="True">
-            <div> <b>&#8594;</b> <a href="${branch.url([ '/bundle', revid, 'bundle.txt' ])}">
-                view/download patch</a> </div>
-        </span>
+        <div> <b>&#8594;</b> <a href="${branch.context_url([ '/files', revid ])}">
+            browse files at revision ${change.revno} </a> </div>
+        <div> <b>&#8594;</b> <a href="${branch.context_url('/changes', start_revid=revid)}">
+            view history from revision ${change.revno} </a> </div>
         <span py:if="compare_revid is not None" py:strip="True">
             <div> <b>&#8594;</b> <a href="${branch.url([ '/bundle', revid, compare_revid, 'bundle.txt' ])}">
-                view/download patch</a> </div>
+                download bundle from ${history.get_revno(compare_revid)} to ${change.revno} </a> </div>
         </span>
-        <span py:if="(remember is not None) and (compare_revid is None)" py:strip="True">
-            <div> <b>&#8594;</b> <a href="${branch.url([ '/revision', revid ],
-                **util.get_context(compare_revid=remember))}">
+        <span py:if="(remember is not None) and (compare_revid is None) and (revid != remember)" py:strip="True">
+            <div> <b>&#8594;</b> <a href="${branch.context_url([ '/revision', revid ], compare_revid=remember)}">
                 compare with revision ${history.get_revno(remember)} </a></div>
         </span>
         <span py:if="remember != revid" py:strip="True">
-            <div> <b>&#8594;</b> <a href="${branch.url([ '/revision', revid ],
-                **util.get_context(remember=revid, compare_revid=None))}">
+            <div> <b>&#8594;</b> <a href="${branch.context_url([ '/revision', revid ], remember=revid, compare_revid=None)}">
                 compare with another revision </a></div>
         </span>
         <span py:if="compare_revid is not None" py:strip="True">
-            <div> <b>&#8594;</b> <a href="${branch.url([ '/revision', revid ],
-                **util.get_context(remember=None, compare_revid=None))}">
-                stop comparing to revision ${history.get_revno(compare_revid)} </a></div>
+            <div> <b>&#8594;</b> <a href="${branch.context_url([ '/revision', compare_revid ], remember=revid, compare_revid=revid)}">
+                reverse the comparison (${change.revno} to ${history.get_revno(compare_revid)})</a></div>
+            <div> <b>&#8594;</b> <a href="${branch.context_url([ '/revision', revid ], remember=None, compare_revid=None)}">
+                stop comparing with revision ${history.get_revno(compare_revid)} </a></div>
         </span>
     </div>
 </h1>
@@ -80,7 +84,7 @@
         </tr>
         <tr>
             <th class="date">date:</th>
-            <td class="date"> ${change.date.strftime('%d %b %Y %H:%M')} </td>
+            <td class="date"> ${util.format_date(change.date)} </td>
         </tr>
 
         <tr py:if="len(change.merge_points) > 0">
@@ -142,9 +146,9 @@
     <td> ${collapse_all_button('file', 'table-row')} </td>
 
     <td class="spacey">
-        <a href="javascript:show_sbs()" class="hide-all collapse-style-sbs-show" title="collapse">
+        <a href="javascript:show_sbs('${branch.url('/')}')" class="hide-all collapse-style-sbs-show" title="collapse">
             <img src="${tg.url('/static/images/nav-small-out.gif')}" width="22" height="10" class="collapse-triangle" />show side by side</a>
-        <a href="javascript:show_unified()" class="hide-all collapse-style-unified-show" title="expand">
+        <a href="javascript:show_unified('${branch.url('/')}')" class="hide-all collapse-style-unified-show" title="expand">
             <img src="${tg.url('/static/images/nav-small-in.gif')}" width="22" height="10" class="collapse-triangle" />show unified diff</a>
     </td>
     

=== modified file 'loggerhead/tests/test_corners.py'
--- loggerhead/tests/test_corners.py	2007-06-27 15:38:47 +0000
+++ loggerhead/tests/test_corners.py	2007-09-10 13:43:45 +0000
@@ -1,22 +1,36 @@
-from loggerhead.tests.test_simple import BasicTests
-
 import os
+
+import cherrypy
 from turbogears import testutil
-import cherrypy
+
+from loggerhead.tests.test_simple import BasicTests
 
 class TestCornerCases(BasicTests):
+    """Tests that excercise various corner cases."""
+
+    def addFileAndCommit(self, filename, commit_msg):
+        """Make a trivial commit that has 'msg' as its commit message.
+
+        The commit adds a file called 'myfilename' containing the string
+        'foo'.
+        """
+        f = open(os.path.join(self.bzrbranch, filename), 'w')
+        try:
+            f.write("foo")
+        finally:
+            f.close()
+        self.tree.add(filename)
+        self.tree.commit(message=commit_msg)
+
 
     def test_survive_over_upgrade(self):
+        """Check that running 'bzr upgrade' on a branch does not break an
+        instance of loggerhead that's already looked at it.
+        """
         self.createBranch()
 
-        f = open(os.path.join(self.bzrbranch, 'myfilename'), 'w')
-        try:
-            f.write("foo")
-        finally:
-            f.close()
-        self.tree.add('myfilename')
         msg = 'a very exciting commit message'
-        self.tree.commit(message=msg)
+        self.addFileAndCommit('myfilename', msg)
 
         self.setUpLoggerhead()
 
@@ -30,24 +44,52 @@
         testutil.create_request('/project/branch/changes')
         assert msg in cherrypy.response.body[0]
 
+
     def test_revision_only_changing_execute_bit(self):
+        """Check that a commit that only changes the execute bit of a file
+        does not break the rendering."""
         self.createBranch()
 
-        f = open(os.path.join(self.bzrbranch, 'myfilename'), 'w')
-        try:
-            f.write("foo")
-        finally:
-            f.close()
-        self.tree.add('myfilename')
+        # Just a commit to have a file to change the execute bit of.
         msg = 'a very exciting commit message'
-        self.tree.commit(message=msg)
+        self.addFileAndCommit('myfilename', msg)
 
+        # Make a commit that changes the execute bit of 'myfilename'.
         os.chmod(os.path.join(self.bzrbranch, 'myfilename'), 0755)
-
         newrevid = self.tree.commit(message='make something executable')
 
+        # Check that it didn't break things.
         self.setUpLoggerhead()
-
-
         testutil.create_request('/project/branch/revision/'+newrevid)
         assert 'executable' in cherrypy.response.body[0]
+
+
+    def test_empty_commit_message(self):
+        """Check that an empty commit message does not break the rendering."""
+        self.createBranch()
+
+        # Make a commit that has an empty message.
+        self.addFileAndCommit('myfilename', '')
+
+        # Check that it didn't break things.
+        self.setUpLoggerhead()
+        testutil.create_request('/project/branch/changes')
+        # It's not much of an assertion, but we only really care about
+        # "assert not crashed".
+        assert 'myfilename' in cherrypy.response.body[0]
+
+
+    def test_whitespace_only_commit_message(self):
+        """Check that a whitespace-only commit message does not break the
+        rendering."""
+        self.createBranch()
+
+        # Make a commit that has a whitespace only message.
+        self.addFileAndCommit('myfilename', '   ')
+
+        # Check that it didn't break things.
+        self.setUpLoggerhead()
+        testutil.create_request('/project/branch/changes')
+        # It's not much of an assertion, but we only really care about
+        # "assert not crashed".
+        assert 'myfilename' in cherrypy.response.body[0]

=== added file 'loggerhead/tests/test_filechangecache.py'
--- loggerhead/tests/test_filechangecache.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/test_filechangecache.py	2007-09-10 12:51:42 +0000
@@ -0,0 +1,95 @@
+"""Tests for the FileChangeCache in loggerhead.changecache."""
+
+import shutil
+import tempfile
+
+from loggerhead.changecache import FileChangeCache
+
+class MockEntry:
+    def __init__(self, revid):
+        self.revid = revid
+
+
+class MockHistory:
+    def __init__(self):
+        self.fetched_revids = set()
+    def get_file_changes_uncached(self, entries):
+        output = []
+        for entry in entries:
+            self.fetched_revids.add(entry.revid)
+            output.append(entry.revid)
+        return output
+
+
+class TestFileChangeCache(object):
+
+    # setup_method and teardown_method are so i can run the tests with
+    # py.test and take advantage of the error reporting.
+    def setup_method(self, meth):
+        self.setUp()
+
+    def teardown_method(self, meth):
+        self.tearDown()
+
+    def setUp(self):
+        self.cache_folders = []
+
+    def tearDown(self):
+        for folder in self.cache_folders:
+            shutil.rmtree(folder)
+
+
+    def makeHistoryAndEntriesForRevids(self, revids, fill_cache_with=[]):
+        cache_folder = tempfile.mkdtemp()
+        self.cache_folders.append(cache_folder)
+        self.history = MockHistory()
+        self.cache = FileChangeCache(self.history, cache_folder)
+
+        self.entries = [MockEntry(revid) for revid in revids]
+
+        self.cache.get_file_changes([entry for entry in self.entries
+                                     if entry.revid in fill_cache_with])
+        self.history.fetched_revids.clear()
+
+
+    def test_empty_cache(self):
+        """An empty cache passes all the revids through to the history object.
+        """
+        revids = ['a', 'b']
+        self.makeHistoryAndEntriesForRevids(revids)
+
+        result = self.cache.get_file_changes(self.entries)
+
+        assert result == revids
+        assert self.history.fetched_revids == set(revids)
+
+    def test_full_cache(self):
+        """A full cache passes none of the revids through to the history
+        object.
+        """
+        revids = ['a', 'b']
+        self.makeHistoryAndEntriesForRevids(revids, fill_cache_with=revids)
+
+        result = self.cache.get_file_changes(self.entries)
+
+        assert result == revids
+        assert self.history.fetched_revids == set()
+
+    def test_partial_cache(self):
+        """A partially full cache passes some of the revids through to the
+        history object, and preserves the ordering of the argument list.
+        """
+        # To test the preservation of argument order code, we put the uncached
+        # revid at the beginning, middle and then end of the list of revids
+        # being asked for.
+        for i in range(3):
+            cached_revids = ['a', 'b']
+            revids = cached_revids[:]
+            revids.insert(i, 'uncached')
+            self.makeHistoryAndEntriesForRevids(
+                revids, fill_cache_with=cached_revids)
+
+            result = self.cache.get_file_changes(self.entries)
+            assert result == revids
+
+            assert self.history.fetched_revids == set(['uncached'])

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2007-06-26 19:23:08 +0000
+++ loggerhead/tests/test_simple.py	2008-02-27 02:41:36 +0000
@@ -23,9 +23,8 @@
 
 class BasicTests(object):
 
-
-    # there are so i can run it with py.test and take advantage of the
-    # error reporting...
+    # setup_method and teardown_method are so i can run the tests with
+    # py.test and take advantage of the error reporting.
     def setup_method(self, meth):
         self.setUp()
 
@@ -85,7 +84,7 @@
     def test_index(self):
         testutil.create_request('/')
         link = '<a href="/project/branch">branch</a>'
-        assert link in cherrypy.response.body[0]
+        assert link in cherrypy.response.body[0].lower()
 
     def test_changes(self):
         testutil.create_request('/project/branch/changes')
@@ -129,7 +128,7 @@
     def test_index(self):
         testutil.create_request('/')
         link = '<a href="/project/branch">branch</a>'
-        assert link in cherrypy.response.body[0]
+        assert link in cherrypy.response.body[0].lower()
 
     def test_changes(self):
         testutil.create_request('/project/branch/changes')

=== modified file 'loggerhead/textindex.py'
--- loggerhead/textindex.py	2007-01-02 07:14:03 +0000
+++ loggerhead/textindex.py	2008-02-27 03:36:09 +0000
@@ -19,22 +19,19 @@
 """
 indexing of the comment text of revisions, for fast searching.
 
-two separate 'shelve' files are created:
+two separate database files are created:
 
     - recorded: revid -> 1 (if the revid is indexed)
     - index: 3-letter substring -> list(revids)
 """
 
-import logging
 import os
 import re
-import shelve
-import threading
 import time
 
 from loggerhead import util
-from loggerhead.util import decorator
 from loggerhead.lockfile import LockFile
+from loggerhead.changecache import FakeShelf
 
 # if any substring index reaches this many revids, replace the entry with
 # an ALL marker -- it's not worth an explicit index.
@@ -65,55 +62,63 @@
     def __init__(self, history, cache_path):
         self.history = history
         self.log = history.log
-        
+
         if not os.path.exists(cache_path):
             os.mkdir(cache_path)
-        
-        self._recorded_filename = os.path.join(cache_path, 'textindex-recorded')
-        self._index_filename = os.path.join(cache_path, 'textindex')
-        
+
+        self._recorded_filename = os.path.join(cache_path, 'textindex-recorded.sql')
+        self._index_filename = os.path.join(cache_path, 'textindex.sql')
+
         # use a lockfile since the cache folder could be shared across different processes.
         self._lock = LockFile(os.path.join(cache_path, 'index-lock'))
         self._closed = False
-        
+
         self.log.info('Using search index; %d entries.', len(self))
-    
+
+    def _index(self):
+        return FakeShelf(self._index_filename)
+
+    def _recorded(self):
+        return FakeShelf(self._recorded_filename)
+
     def _is_indexed(self, revid, recorded):
-        return recorded.get(util.to_utf8(revid), None) is not None
-        
+        return recorded.get(util.to_utf8(revid)) is not None
+
     @with_lock
     def is_indexed(self, revid):
-        recorded = shelve.open(self._recorded_filename, 'c', protocol=2)
+        recorded = self._recorded()
         try:
             return self._is_indexed(revid, recorded)
         finally:
             recorded.close()
-    
+
     @with_lock
     def __len__(self):
-        recorded = shelve.open(self._recorded_filename, 'c', protocol=2)
+        recorded = self._recorded()
         try:
-            return len(recorded)
+            return recorded.count()
         finally:
             recorded.close()
 
     @with_lock
     def close(self):
         self._closed = True
-    
+
     @with_lock
     def closed(self):
         return self._closed
-    
+
     @with_lock
     def flush(self):
         pass
-    
+
     @with_lock
     def full(self):
-        recorded = shelve.open(self._recorded_filename, 'c', protocol=2)
+        recorded = self._recorded()
+        last_revid = util.to_utf8(self.history.last_revid)
         try:
-            return (len(recorded) >= len(self.history.get_revision_history())) and (util.to_utf8(self.history.last_revid) in recorded)
+            return (recorded.count() >= len(self.history.get_revision_history())
+                    and recorded.get(last_revid) is not None)
         finally:
             recorded.close()
 
@@ -126,7 +131,7 @@
             return
         for i in xrange(len(comment) - 2):
             sub = comment[i:i + 3]
-            revid_set = index.get(sub, None)
+            orig = revid_set = index.get(sub)
             if revid_set is None:
                 revid_set = set()
             elif revid_set == ALL:
@@ -135,39 +140,42 @@
             revid_set.add(change.revid)
             if len(revid_set) > ALL_THRESHOLD:
                 revid_set = ALL
-            index[sub] = revid_set
-        
-        recorded[util.to_utf8(change.revid)] = True
+            if orig is not None:
+                index.update([(sub, revid_set)], commit=False)
+            else:
+                index.add([(sub, revid_set)], commit=False)
+
+        recorded.add([(util.to_utf8(change.revid), True)], commit=False)
 
     @with_lock
     def index_changes(self, revid_list):
-        recorded = shelve.open(self._recorded_filename, 'c', protocol=2)
-        index = shelve.open(self._index_filename, 'c', protocol=2)
+        recorded = self._recorded()
+        index = self._index()
         try:
             revid_list = [r for r in revid_list if not self._is_indexed(r, recorded)]
             change_list = self.history.get_changes(revid_list)
             for change in change_list:
                 self._index_change(change, recorded, index)
         finally:
-            index.close()
-            recorded.close()
-    
+            index.close(commit=True)
+            recorded.close(commit=True)
+
     @with_lock
     def find(self, text, revid_list=None):
-        index = shelve.open(self._index_filename, 'c', protocol=2)
+        index = self._index()
         try:
             text = normalize_string(text)
             if len(text) < 3:
                 return []
-    
+
             total_set = None
             if revid_list is not None:
                 total_set = set(revid_list)
             seen_all = False
-            
+
             for i in xrange(len(text) - 2):
                 sub = text[i:i + 3]
-                revid_set = index.get(sub, None)
+                revid_set = index.get(sub)
                 if revid_set is None:
                     # zero matches, stop here.
                     return []
@@ -183,7 +191,7 @@
                     return []
         finally:
             index.close()
-        
+
         # tricky: if seen_all is True, one of the substring indices was ALL
         # (in other words, unindexed), so our results are actually a superset
         # of the exact answer.
@@ -193,7 +201,7 @@
         # we DON'T care, and if one of the substrings hit ALL, there's a small
         # chance that we'll give a few false positives.
         return total_set
-    
+
     def check_rebuild(self, max_time=3600):
         """
         check if there are any un-indexed revisions, and if so, index them.
@@ -230,4 +238,3 @@
             time.sleep(1)
         self.log.info('Search index completed.')
         self.flush()
-

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2007-06-25 06:44:52 +0000
+++ loggerhead/util.py	2008-02-27 03:36:09 +0000
@@ -29,8 +29,6 @@
 import time
 import traceback
 
-import turbogears
-
 
 log = logging.getLogger("loggerhead.controllers")
 
@@ -78,6 +76,25 @@
     return year
 
 
+_g_format = '%Y-%m-%d @ %H:%M'
+
+def format_date(date):
+    if _g_format == 'fancy':
+        return fancy_format_date(date)
+    return date.strftime(_g_format)
+
+def fancy_format_date(date):
+    delta = datetime.datetime.now() - date
+    if delta.days > 300:
+        return date.strftime('%d %b %Y')
+    else:
+        return date.strftime('%d %b %H:%M')
+
+def set_date_format(format):
+    global _g_format
+    _g_format = format
+
+
 class Container (object):
     """
     Convert a dict into an object with attributes.
@@ -88,7 +105,7 @@
                 setattr(self, key, value)
         for key, value in kw.iteritems():
             setattr(self, key, value)
-    
+
     def __repr__(self):
         out = '{ '
         for key, value in self.__dict__.iteritems():
@@ -143,7 +160,7 @@
         return '%s at %s' % (username, domains[-2])
     return '%s at %s' % (username, domains[0])
 
-    
+
 def triple_factors(min_value=1):
     factors = (1, 3)
     index = 0
@@ -161,10 +178,10 @@
     """
     given a position in a maximum range, return a list of negative and positive
     jump factors for an hgweb-style triple-factor geometric scan.
-    
+
     for example, with pos=20 and max=500, the range would be:
     [ -10, -3, -1, 1, 3, 10, 30, 100, 300 ]
-    
+
     i admit this is a very strange way of jumping through revisions.  i didn't
     invent it. :)
     """
@@ -272,7 +289,7 @@
         divisor = MEG
     else:
         divisor = KILO
-    
+
     dot = size % divisor
     base = size - dot
     dot = dot * 10 // divisor
@@ -280,7 +297,7 @@
     if dot >= 10:
         base += 1
         dot -= 10
-    
+
     out = str(base)
     if (base < 100) and (dot != 0):
         out += '.%d' % (dot,)
@@ -291,7 +308,7 @@
     elif divisor == GIG:
         out += 'G'
     return out
-    
+
 
 def fill_in_navigation(navigation):
     """
@@ -305,21 +322,21 @@
     navigation.count = len(navigation.revid_list)
     navigation.page_position = navigation.position // navigation.pagesize + 1
     navigation.page_count = (len(navigation.revid_list) + (navigation.pagesize - 1)) // navigation.pagesize
-    
+
     def get_offset(offset):
         if (navigation.position + offset < 0) or (navigation.position + offset > navigation.count - 1):
             return None
         return navigation.revid_list[navigation.position + offset]
-    
+
     navigation.prev_page_revid = get_offset(-1 * navigation.pagesize)
     navigation.next_page_revid = get_offset(1 * navigation.pagesize)
-    
+
     params = { 'file_id': navigation.file_id }
     if getattr(navigation, 'query', None) is not None:
         params['q'] = navigation.query
     else:
         params['start_revid'] = navigation.start_revid
-        
+
     if navigation.prev_page_revid:
         navigation.prev_page_url = navigation.branch.url([ navigation.scan_url, navigation.prev_page_revid ], **get_context(**params))
     if navigation.next_page_revid:
@@ -432,7 +449,7 @@
     """
     return a context map that may be overriden by specific values passed in,
     but only contains keys from the list of valid context keys.
-    
+
     if 'clear' is set, only the 'remember' context value will be added, and
     all other context will be omitted.
     """

=== modified file 'start-loggerhead.py'
--- start-loggerhead.py	2007-06-19 00:09:43 +0000
+++ start-loggerhead.py	2007-09-04 17:06:03 +0000
@@ -12,7 +12,7 @@
 import cherrypy
 cherrypy.lowercase_api = True
 
-from loggerhead import daemon, release
+from loggerhead import daemon, release, util
 
 
 def make_handler(config, filename):
@@ -111,10 +111,10 @@
     Root = Root(config)
     
     # re-index every 6 hours
-    
     index_freq = config.get('cache_rebuild_frequency', 6 * 3600)
     turbogears.scheduler.add_interval_task(initialdelay=1, interval=index_freq, action=Root._check_rebuild)
-    
+
+    util.set_date_format(config.get('date_format', 'fancy'))
     try:
         turbogears.start_server(Root)
     finally:

=== removed directory 'turbozpt'
=== removed file 'turbozpt-README.txt'
--- turbozpt-README.txt	2007-06-29 10:47:51 +0000
+++ turbozpt-README.txt	1970-01-01 00:00:00 +0000
@@ -1,17 +0,0 @@
-TurboZpt
-========
-
-This package provides a TurboGears template engine plugin, allowing you
-to easily use Zope Page Templates with TurboGears.
-
-Zope Page Templates are assumed to have a "pt" extension.
-
-For information on using a TurboGears template plugin, go here:
-
-http://www.turbogears.org/docs/plugins/template.html
-
-At this moment TurboZpt does not do everything KID does. (but neither
-does TurboCheetah as far as i'm awhere)
-
-It's far from final and hopefully see lots of intresting updates in the
-feature. Hopefully as part of the standard set of TurboGears Plugins. :)

=== removed file 'turbozpt-setup.py'
--- turbozpt-setup.py	2007-06-29 10:47:51 +0000
+++ turbozpt-setup.py	1970-01-01 00:00:00 +0000
@@ -1,31 +0,0 @@
-from setuptools import setup, find_packages
-from turbogears.finddata import find_package_data
-
-setup(
-    name="TurboZpt",
-    version="0.1.4",
-    description="TurboGears plugin to support use of Zope Page Templates",
-    author="Bas van Oostveen",
-    author_email="v.oostveen@gmail.com",
-    url="http://ido.nl.eu.org/turbozpt/",
-    download_url="http://ido.nl.eu.org/turbozpt/TurboZpt-0.1.2.tar.gz",
-    license="MIT",
-#    install_requires = ["TurboGears >= 0.9a0dev", "ZopePageTemplates"],
-    install_requires = ["TurboGears >= 0.9a0dev"],
-    zip_safe=False,
-    packages=find_packages(),
-    classifiers = [
-        'Development Status :: 3 - Alpha',
-        #'Environment :: TurboGears',
-        'Operating System :: OS Independent',
-        'Programming Language :: Python',
-        'License :: OSI Approved :: MIT License',
-        'Topic :: Software Development :: Libraries :: Python Modules'
-    ],
-    entry_points="""
-    [python.templating.engines]
-    zpt = turbozpt.zptsupport:TurboZpt
-    """,
-    test_suite = 'nose.collector',
-    )
-    

=== removed file 'turbozpt/__init__.py'
--- turbozpt/__init__.py	2007-06-29 10:47:51 +0000
+++ turbozpt/__init__.py	1970-01-01 00:00:00 +0000
@@ -1,9 +0,0 @@
-from turbozpt import zptsupport
-
-TurboZpt = zptsupport.TurboZpt
-
-__all__ = ["TurboZpt"]
-
-from turbogears.view import engines, stdvars
-
-engines['zpt'] = TurboZpt(stdvars, {})

=== removed file 'turbozpt/template.py'
--- turbozpt/template.py	2007-07-05 09:10:39 +0000
+++ turbozpt/template.py	1970-01-01 00:00:00 +0000
@@ -1,43 +0,0 @@
-"""
-By VladDrac@irc.freenode.net/#turbogears
-+ Some small modifications
-"""
-
-from zope.pagetemplate import pagetemplatefile
-import os.path
-import sys
-
-class Here(object):
-    def __init__(self, base, options):
-        self.base = base
-        self.options = options
-
-    def __getitem__(self, name):
-        tpl = PageTemplate(os.path.join(self.base, name))
-        tpl.add_context(self.options)
-        return tpl
-
-class PageTemplate(pagetemplatefile.PageTemplateFile):
-    def __init__(self, name):
-    	base = os.path.dirname(sys._getframe(1).f_globals["__file__"])
-	self.extra_context = {}
-        self.name = name
-        self.fullpath = os.path.join(base, self.name)
-        self.base = os.path.dirname(self.fullpath)
-        pagetemplatefile.PageTemplateFile.__init__(self, self.fullpath)
-    
-    def render(self, extra_dict=None):
-	if extra_dict:
-    	    context = self.pt_getContext()
-	    context.update(extra_dict)
-	return self.pt_render(context)
-    
-    def add_context(self, d):
-	self.extra_context.update(d)
-	
-    def pt_getContext(self, args=(), options={}, **ignored):
-        rval = pagetemplatefile.PageTemplateFile.pt_getContext(self, args, options, **ignored)
-	rval.update(options)
-	rval.update(self.extra_context)
-        rval.update({'here':Here(self.base, options), 'template':self})
-        return rval

=== removed directory 'turbozpt/tests'
=== removed file 'turbozpt/tests/__init__.py'
=== removed file 'turbozpt/tests/simple.pt'
--- turbozpt/tests/simple.pt	2007-06-29 10:47:51 +0000
+++ turbozpt/tests/simple.pt	1970-01-01 00:00:00 +0000
@@ -1,8 +0,0 @@
-<html>
-<head>
-<title tal:content="title">simple test page title</title>
-</head>
-<body>
-<div>Hello, <span tal:replace="name">name</span></div>
-</body>
-</html>

=== removed file 'turbozpt/tests/test_zpt.py'
--- turbozpt/tests/test_zpt.py	2007-06-29 10:47:51 +0000
+++ turbozpt/tests/test_zpt.py	1970-01-01 00:00:00 +0000
@@ -1,30 +0,0 @@
-from turbozpt import TurboZpt
-from turbogears import controllers
-from turbogears import testutil
-import turbogears
-import cherrypy
-
-RENDERED="<html>\n<head>\n<title>%s</title>\n</head>\n<body>\n<div>Hello, %s</div>\n</body>\n</html>\n"
-
-def test_template_lookup():
-    tc = TurboZpt()
-    template = tc.load_template("turbozpt.tests.simple")
-    assert template
-    TITLE="test"
-    NAME="World"
-    info = dict(title=TITLE, name=NAME)
-    t = template(**info)
-    assert str(t).startswith(RENDERED % (TITLE, NAME))
-
-class TestRoot(controllers.Root):
-    def index(self, name, title="test"):
-        return dict(name=name, title=title)
-    index = turbogears.expose(html="zpt:turbozpt.tests.simple")(index)
-
-def test_real_life_situation():
-    cherrypy.root = TestRoot()
-    TITLE="test dave"
-    NAME="Dave"
-    testutil.createRequest("/?name=%s" % NAME)
-    print cherrypy.response.body
-    assert cherrypy.response.body[0].startswith(RENDERED % ("test", NAME))

=== removed file 'turbozpt/zptsupport.py'
--- turbozpt/zptsupport.py	2007-07-04 15:55:56 +0000
+++ turbozpt/zptsupport.py	1970-01-01 00:00:00 +0000
@@ -1,95 +0,0 @@
-"TurboGears support for Zope Page Templates"
-
-import sys, os
-
-from template import PageTemplate
-import pkg_resources
-
-import logging
-log = logging.getLogger("turbogears.zptsupport")
-
-def _recompile_template(package, basename, tfile, classname):
-    log.debug("Recompiling template for %s" % classname)
-    mod = PageTemplate(tfile)
-    mtime = os.stat(tfile).st_mtime
-    mod.__mtime__ = mtime
-    return mod
-
-class TurboZpt:
-    extension = "pt"
-    
-    def __init__(self, extra_vars_func=None, options={}):
-	self.options = options
-	self.get_extra_vars = extra_vars_func
-        self.compiledTemplates = {}
-    
-    def load_template(self, classname, loadingSite=False):
-        """Searches for a template along the Python path.
-
-        Template files must end in ".pt" and be in legitimate packages.
-        U can set "zpt.cache_templates" option to cache a loaded template
-	class and only check for updates. Templates are automatically
-	checked for changes and reloaded as neccessary.
-        """
-        ct = self.compiledTemplates
-
-        divider = classname.rfind(".")
-        if divider > -1:
-            package = classname[0:divider]
-            basename = classname[divider+1:]
-        else:
-            raise ValueError, "All templates must be in a package"
-
-        cache_templates = self.options.get("zpt.cache_templates", True)
-        tfile = pkg_resources.resource_filename(package, 
-                                                "%s.%s" % 
-                                                (basename,
-                                                self.extension))
-						
-        if cache_templates:
-            if ct.has_key(classname):
-                mtime = os.stat(tfile).st_mtime
-		mod = ct[classname]
-                if mod.__mtime__ != mtime:
-		    # Recompile template
-		    mod = _recompile_template(package, basename, tfile, classname)
-                    ct[classname] = mod
-                #else:
-		    # use template from cache    
-		    # pass 
-            else:
-		# First time compile template
-		mod = PageTemplate(tfile)
-                mod.__mtime__ = os.stat(tfile).st_mtime
-		ct[classname] = mod
-        else:
-	    mod = PageTemplate(tfile)
-
-        return mod
-
-    def render(self, info, format="html", fragment=False, template=None):
-	"""Renders data in the desired format.
-	
-	@param info: the data / context itself
-	@type info: dict
-	@para format: "html"
-	@type format: "string"
-	@para template: name of the template to use
-	@type template: string
-	"""
-	#if isinstance(template, type):
-	#    tclass = template
-	#else:
-	
-	tinstance = self.load_template(template)
-	log.debug("Applying template %s" % (tinstance.filename))
-	data = dict()
-	if self.get_extra_vars:
-	    data.update(self.get_extra_vars())
-	data.update(info)
-        print '!!!!', data.keys()
-        return tinstance(**data)
-
-    def transform(self, info, template):
-	"Render the output to Elements"
-	pass

